(* ::Package:: *)

(* 	
	MAMBO - software for modeling and analysis of multibody systems    
	Copyright 2003-2013, Harry Dankowicz    

	Department of Mechanical Science and Engineering 	
	University of Illinois at Urbana-Champaign   	

	MAMBO Toolbox    
	Version 2.3, compatible with Mathematica 10.3  	

	Revision date: December 18, 2015 
*)  

BeginPackage["MamboToolbox`"];

(* Usage statements *)
Restart::usage="This procedure resets all global variables.";
Undo::usage="This procedure undoes the last change to the global variables.";  

GlobalPointDeclarations = {}; GlobalTriadDeclarations = {}; GlobalObserverDeclarations = {};
GlobalPointDeclarationsOld = {}; GlobalTriadDeclarationsOld = {}; GlobalObserverDeclarationsOld = {};
GlobalPointDefinitions = {}; GlobalTriadDefinitions = {}; GlobalObserverDefinitions = {}; GlobalObjectDefinitions = {};
GlobalPointDefinitionsOld = {}; GlobalTriadDefinitionsOld = {}; GlobalObserverDefinitionsOld = {}; GlobalObjectDefinitionsOld = {};
GlobalAngularVelocity = {}; GlobalStates = {};

DeclarePoints::usage="DeclarePoints[A,B,...] is used to declare a sequence of points.";
DeclareTriads::usage="DeclareTriads[a,b,...] is used to declare a sequence of triads.";
DeclareObservers::usage="DeclareObservers[A,B,...] is used to declare a sequence of observers.";
DefinePoints::usage="DefinePoints[A,B,...] is used to define a sequence of points.";
DefineTriads::usage="DefineTriads[A,B,...] is used to define a sequence of triads.";
DefineObservers::usage="DefineObservers[A,B,...] is used to define a sequence of observers.";
DefineNeighbors::usage="DefineNeighbors[A,B,...] is used to define neighbors of observers.";
DefineObjects::usage="DefineObjects[] is  used to associated objects with observers.";
FindRotation::usage="";
FindOrientation::usage="";
FindTranslation::usage="";
FindPosition::usage="";
FindCoordinates::usage="";

SetTime::usage="";
DiffTime::usage="";
AngularVelocity::usage="";
LinearVelocity::usage="";
LinearMomentum::usage="";
AngularMomentum::usage=""; 

NullVector::usage="NullVector is used to generate a zero vector.";
VelocityDescription::usage=" ";
MomentumDescription::usage=" ";
ForceDescription::usage=" ";
MakeDescription::usage=" ";
CoeffExtract::usage=""; 
MakeTranslations::usage=" ";
MakeRotations::usage=" ";
MamboPlus::usage="Addition of Mambo vectors or descriptions";
MamboTimes::usage="Multiplication of Mambo vectors or descriptions by a scalar";
MamboMinus::usage="Substraction of Mambo vectors of descriptions or unary negative of a Mambo vector or description";
MamboDot::usage="Dot product of Mambo vectors or descriptions";
MamboNorm::usage="Vector norm of Mambo vectors";
MamboCross::usage="Cross product of Mambo vectors";
MamboExpress::usage="Express Mambo vector in given triad";
MamboSimplify::usage="Simplify Mambo vector or description";

DeclareStates::usage="";

GeometryOutput::usage="";
MotionOutput::usage="";

MamboDescriptionQ::usage="";
RuleQ::usage="";

(* Warning messages *)
DeclareName::MT1="Object `1` already declared!";
DeclareName::MT2="Object `1` is undeclared!";
DeclareName::MT3="Path already exists between `1` and `2`!";
DeclareName::MT4="Object `1` already defined!";
DeclareName::MT5="Reference point `1` is undeclared!";
DeclareName::MT6="Reference triad `1` is undeclared!";
DeclareName::MT7="Path does not exist between `1` and `2`!";
DeclareName::MT8="Object `1` is not defined!";
DeclareName::MT9="Variable `1` not declared!";
MamboToolbox::IA="Incompatible arguments in `1`";
MamboToolbox::NO="Number of odes is not equal to number of states"; 
MamboToolbox::SM="Singular mass matrix"; 

GeometryOutput::GO1="No main observer declared!";  
GeometryOutput::GO2="Incomplete geometry!";  

(* Options *)  

(* Private context *) 
 Begin["`Private`"];  

(* GENERAL UTILITIES *)  
RuleQ[lis_]:= If[ListQ[lis],Apply[And,Map[SameQ[Head[#],Rule]&,lis]],False];  

(* TREE UTILITIES *)  
Restart := ( 	
	GlobalPointDeclarations = {}; 	GlobalTriadDeclarations = {}; GlobalObserverDeclarations = {};
	GlobalPointDeclarationsOld = {}; GlobalTriadDeclarationsOld = {}; GlobalObserverDeclarationsOld = {}; 
	GlobalPointDefinitions = {}; GlobalTriadDefinitions = {}; GlobalObserverDefinitions = {}; GlobalObjectDefinitions = {}; 	
	GlobalPointDefinitionsOld = {}; GlobalTriadDefinitionsOld = {}; GlobalObserverDefinitionsOld = {}; GlobalObjectDefinitionsOld = {}; 	
	GlobalAngularVelocity = {}; GlobalStates = {}; t=.; 
)  

Backup := ( 	
	GlobalPointDeclarationsOld = GlobalPointDeclarations; GlobalTriadDeclarationsOld = GlobalTriadDeclarations; 	
	GlobalObserverDeclarationsOld = GlobalObserverDeclarations; GlobalPointDefinitionsOld = GlobalPointDefinitions; 	
	GlobalTriadDefinitionsOld = GlobalTriadDefinitions; GlobalObserverDefinitionsOld = GlobalObserverDefinitions; 	
	GlobalObjectDefinitionsOld = GlobalObjectDefinitions; 
)  

Undo := ( 	
	GlobalPointDeclarations = GlobalPointDeclarationsOld; GlobalTriadDeclarations = GlobalTriadDeclarationsOld; 	
	GlobalObserverDeclarations = GlobalObserverDeclarationsOld; GlobalPointDefinitions = GlobalPointDefinitionsOld; 	
	GlobalTriadDefinitions = GlobalTriadDefinitionsOld; GlobalObserverDefinitions = GlobalObserverDefinitionsOld; 	
	GlobalObjectDefinitions = GlobalObjectDefinitionsOld; 	
	Print["Last change to global variables has been undone!"]; 
)  
MamboFindPath[name1_,name2_,net_] := Module[{path={},subnet,i=1}, 	
	If[NameQ[ToString[name1]] && NameQ[ToString[name2]] && ListQ[name1/.net] && ListQ[name2/.net], 		
	If[SameQ[name1,name2],Return[name1]]; 		
	If[MemberQ[name1/.net,name2],Return[{name1,name2}]]; 		
	subnet=Delete[net,Position[net,name1->_]]; 		
	While[i<=Length[name1/.net],(path=MamboFindPath[#,name2,subnet];If[path!={},Return[Prepend[path,name1]]])&[(name1/.net)[[i]]];i++]; 	]; 	
	Return[{}]; 
]  

(* GEOMETRY UTILITIES *)  
DeclarePoints[a__] := (
	Backup; 	
	Map[( 		
		If[Not[NameQ[ToString[#]]],(Message[MamboToolbox::IA,"DeclarePoints"];Undo;Abort[])]; 		
		If[ListQ[#/.GlobalPointDeclarations],(Message[DeclareName::MT1,#];Undo;Abort[])]; 		
		AppendTo[GlobalPointDeclarations,#->{}]; 	
	)&,{a}]; 
) 

DeclareTriads[a__] := (
	Backup; 	
	Map[( 		
		If[Not[NameQ[ToString[#]]],(Message[MamboToolbox::IA,"DeclareTriads"];Undo;Abort[])]; 		
		If[ListQ[#/.GlobalTriadDeclarations],(Message[DeclareName::MT1,#];Undo;Abort[])]; 		
		AppendTo[GlobalTriadDeclarations,#->{}]; 	
	)&,{a}]; 
) 

DeclareObservers[a__] := (
	Backup; 	
	Map[( 		
		If[Not[NameQ[ToString[#]]],(Message[MamboToolbox::IA,"DeclareObservers"];Undo;Abort[])]; 		
		If[ListQ[#/.GlobalObserverDeclarations],(Message[DeclareName::MT1,#];Undo;Abort[])]; 		
		AppendTo[GlobalObserverDeclarations,#->{}]; 		
		AppendTo[GlobalObjectDefinitions,#->{}]; 	
	)&,{a}]; 
) 

DefinePoints[a__] := (
	Backup; 	
	If[Apply[And,Map[ListQ,{a}]], 		
		Map[( 			
			If[Length[#]==1 || Not[NameQ[ToString[#[[1]]]]] ||  Not[NameQ[ToString[#[[2]]]]],(Message[MamboToolbox::IA,"DefinePoints"];Undo;Abort[])]; 
			If[Not[ListQ[#[[1]]/.GlobalPointDeclarations]],(Message[DeclareName::MT2,#[[1]]];Undo;Abort[])];
 			If[Not[ListQ[#[[2]]/.GlobalPointDeclarations]],(Message[DeclareName::MT2,#[[2]]];Undo;Abort[])];
 			If[Length[MamboFindPath[#[[1]],#[[2]],GlobalPointDeclarations]]>0,(Message[DeclareName::MT3,#[[1]],#[[2]]];Undo;Abort[])];
 			GlobalPointDeclarations=ReplacePart[GlobalPointDeclarations,#[[1]]->Append[#[[1]]/.GlobalPointDeclarations,#[[2]]],Position[GlobalPointDeclarations,#[[1]]->_]];
 			GlobalPointDeclarations=ReplacePart[GlobalPointDeclarations,#[[2]]->Append[#[[2]]/.GlobalPointDeclarations,#[[1]]],Position[GlobalPointDeclarations,#[[2]]->_]];
 			AppendTo[GlobalPointDefinitions,{#[[1]],#[[2]]}->MakeTranslations@@Take[#,{3,Length[#]}]];
 			AppendTo[GlobalPointDefinitions,{#[[2]],#[[1]]}->MamboMinus[{#[[1]],#[[2]]}/.GlobalPointDefinitions]];
 		)&,{a}],
 	DefinePoints[{a}]];
)

DefineTriads[a__] := (
	Backup;
 	If[Apply[And,Map[ListQ,{a}]],
 		Map[(
 			If[Length[#]==1 || Not[NameQ[ToString[#[[1]]]]] ||  Not[NameQ[ToString[#[[2]]]]],(Message[MamboToolbox::IA,"DefineTriads"];Undo;Abort[])];
 			If[Not[ListQ[#[[1]]/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,#[[1]]];Undo;Abort[])];
 			If[Not[ListQ[#[[2]]/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,#[[2]]];Undo;Abort[])];
 			If[Length[MamboFindPath[#[[1]],#[[2]],GlobalTriadDeclarations]]>0,(Message[DeclareName::MT3,#[[1]],#[[2]]];Undo;Abort[])];
 			GlobalTriadDeclarations=ReplacePart[GlobalTriadDeclarations,#[[1]]->Append[#[[1]]/.GlobalTriadDeclarations,#[[2]]],Position[GlobalTriadDeclarations,#[[1]]->_]];
 			GlobalTriadDeclarations=ReplacePart[GlobalTriadDeclarations,#[[2]]->Append[#[[2]]/.GlobalTriadDeclarations,#[[1]]],Position[GlobalTriadDeclarations,#[[2]]->_]];
 			AppendTo[GlobalTriadDefinitions,{#[[1]],#[[2]]}->MakeRotations@@Take[#,{3,Length[#]}]];
 			AppendTo[GlobalTriadDefinitions,{#[[2]],#[[1]]}->Transpose[{#[[1]],#[[2]]}/.GlobalTriadDefinitions]];
 		)&,{a}],
 	DefineTriads[{a}]];
)

DefineObservers[a__] := (
	Backup;
 	If[Apply[And,Map[ListQ,{a}]],
 		Map[(
 			If[Length[#]!=3 || Not[NameQ[ToString[#[[1]]]]] ||  Not[NameQ[ToString[#[[2]]]]] ||  Not[NameQ[ToString[#[[3]]]]],(Message[MamboToolbox::IA,"DefineObservers"];Undo;Abort[])]; 
			If[Not[ListQ[#[[1]]/.GlobalObserverDeclarations]],(Message[DeclareName::MT2,#[[1]]];Undo;Abort[])];
 			If[ListQ[#[[1]]/.GlobalObserverDefinitions],(Message[DeclareName::MT4,#[[2]]];Undo;Abort[])];
 			If[Not[ListQ[#[[2]]/.GlobalPointDeclarations]],(Message[DeclareName::MT5,#[[2]]];Undo;Abort[])];
 			If[Not[ListQ[#[[3]]/.GlobalTriadDeclarations]],(Message[DeclareName::MT6,#[[3]]];Undo;Abort[])];
 			AppendTo[GlobalObserverDefinitions,#[[1]]->{"Point"->#[[2]],"Triad"->#[[3]]}]; 
		)&,{a}],
 	DefineObservers[{a}]];
)

DefineNeighbors[a__] := (
	Backup;
 	If[Apply[And,Map[ListQ,{a}]], 
		Map[( 
			If[Length[#]!=2 || Not[NameQ[ToString[#[[1]]]]] ||  Not[NameQ[ToString[#[[2]]]]],(Message[MamboToolbox::IA,"DefineNeighbors"];Undo;Abort[])];
 			If[Not[ListQ[#[[1]]/.GlobalObserverDeclarations]],(Message[DeclareName::MT2,#[[1]]];Undo;Abort[])]; 
			If[Not[ListQ[#[[2]]/.GlobalObserverDeclarations]],(Message[DeclareName::MT2,#[[2]]];Undo;Abort[])]; 
			If[Length[MamboFindPath[#[[1]],#[[2]],GlobalObserverDeclarations]]>0,(Message[DeclareName::MT3,#[[1]],#[[2]]];Undo;Abort[])];
 			GlobalObserverDeclarations=ReplacePart[GlobalObserverDeclarations,#[[1]]->Append[#[[1]]/.GlobalObserverDeclarations,#[[2]]],Position[GlobalObserverDeclarations,#[[1]]->_]];
 			GlobalObserverDeclarations=ReplacePart[GlobalObserverDeclarations,#[[2]]->Append[#[[2]]/.GlobalObserverDeclarations,#[[1]]],Position[GlobalObserverDeclarations,#[[2]]->_]];
 		)&,{a}],
 	DefineNeighbors[{a}]];
)

FindRotation[triad1_,triad2_] := Module[{path,conn},
 	If[Not[NameQ[ToString[triad1]]] || Not[NameQ[ToString[triad2]]],(Message[MamboToolbox::IA,"FindRotation"];Abort[])];
 	If[Not[ListQ[triad1/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,triad1];Abort[])];
 	If[Not[ListQ[triad2/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,triad2];Abort[])]; 
	If[SameQ[triad1,triad2],Return[IdentityMatrix[3]]];
 	If[MatrixQ[{triad1,triad2}/.GlobalTriadDefinitions],Return[{triad1,triad2}/.GlobalTriadDefinitions]];
 	path=MamboFindPath[triad1,triad2,GlobalTriadDeclarations];
 	If[path=={},(Message[DeclareName::MT7,triad1,triad2];Abort[])]; 
	Do[If[Not[MatrixQ[triad1,path[[conn]]]],
 		(GlobalTriadDefinitions=Union[GlobalTriadDefinitions, 
			{{triad1,path[[conn]]}->Dot[{triad1,path[[conn-1]]}/.GlobalTriadDefinitions,{path[[conn-1]],path[[conn]]}/.GlobalTriadDefinitions]}]; 
		GlobalTriadDefinitions=Union[GlobalTriadDefinitions, 	
		{{path[[conn]],triad1}->Transpose[{triad1,path[[conn]]}/.GlobalTriadDefinitions]}])],
 	{conn,3,Length[path]}];
 	Return[{triad1,triad2}/.GlobalTriadDefinitions];
]

FindOrientation[observer1_,observer2_] := (
 	If[Not[NameQ[ToString[observer1]]] || Not[NameQ[ToString[observer2]]],(Message[MamboToolbox::IA,"FindOrientation"];Abort[])];
 	If[Not[ListQ[observer1/.GlobalObserverDefinitions]],(Message[DeclareName::MT8,observer1];Abort[])];
 	If[Not[ListQ[observer2/.GlobalObserverDefinitions]],(Message[DeclareName::MT8,observer2];Abort[])];
 	Return[FindRotation["Triad"/.(observer1/.GlobalObserverDefinitions),"Triad"/.(observer2/.GlobalObserverDefinitions)]];
)

FindTranslation[point1_,point2_] := Module[{path,conn},
 	If[Not[NameQ[ToString[point1]]] || Not[NameQ[ToString[point2]]],(Message[MamboToolbox::IA,"FindTranslation"];Abort[])];
 	If[Not[ListQ[point1/.GlobalPointDeclarations]],(Message[DeclareName::MT2,point1];Abort[])];
 	If[Not[ListQ[point2/.GlobalPointDeclarations]],(Message[DeclareName::MT2,point2];Abort[])];
 	If[SameQ[point1,point2],Return[NullVector]];
 	If[MamboVectorQ[{point1,point2}/.GlobalPointDefinitions],Return[{point1,point2}/.GlobalPointDefinitions]];
 	path=MamboFindPath[point1,point2,GlobalPointDeclarations];
 	If[path=={},(Message[DeclareName::MT7,point1,point2];Abort[])]; 
	Do[If[Not[MamboVectorQ[{point1,path[[conn]]}/.GlobalPointDefinitions]], 
		(GlobalPointDefinitions=Union[GlobalPointDefinitions, 
			{{point1,path[[conn]]}->MamboPlus[{point1,path[[conn-1]]}/.GlobalPointDefinitions,{path[[conn-1]],path[[conn]]}/.GlobalPointDefinitions]}]; 
		GlobalPointDefinitions=Union[GlobalPointDefinitions, 
			{{path[[conn]],point1}->MamboMinus[{point1,path[[conn]]}/.GlobalPointDefinitions]}])], 
	{conn,3,Length[path]}]; 
	Return[{point1,point2}/.GlobalPointDefinitions];
]

FindPosition[observer1_,observer2_] := (
 	If[Not[NameQ[ToString[observer1]]] || Not[NameQ[ToString[observer2]]],(Message[MamboToolbox::IA,"FindPosition"];Abort[])];
 	If[Not[ListQ[observer1/.GlobalObserverDefinitions]],(Message[DeclareName::MT8,observer1];Abort[])];
 	If[Not[ListQ[observer2/.GlobalObserverDefinitions]],(Message[DeclareName::MT8,observer2];Abort[])];
 	Return[FindTranslation["Point"/.(observer1/.GlobalObserverDefinitions),"Point"/.(observer2/.GlobalObserverDefinitions)]];
)

FindCoordinates[observer_,point_] := Module[{localev},
 	If[Not[NameQ[ToString[observer]]] || Not[NameQ[ToString[point]]],(Message[MamboToolbox::IA,"FindCoordinates"];Abort[])];
 	If[Not[ListQ[observer/.GlobalObserverDefinitions]],(Message[DeclareName::MT8,observer1];Abort[])];
 	localev=FindTranslation["Point"/.(observer/.GlobalObserverDefinitions),point];
 	If[("Size"/.localev)==0,Return[{0,0,0}],Return["Coordinates"/.(1/.MamboExpress[localev,"Triad"/.(observer/.GlobalObserverDefinitions)])]];
]

(* Translations and Rotations *)  
MamboVectorQ[ev_] := If[RuleQ[ev],Equal["Type"/.ev,"Vector"],False]  
MamboDescriptionQ[ev_] := If[RuleQ[ev],Equal["Type"/.ev,"Description"],False]  
NullVector := {"Type"->"Vector","Size"->0}  

MakeDescription[ev_,rows_] := Module[{cols=Quotient[Length[ev],rows],localev={}},
 	If[Length[ev]-cols*rows!=0,(Message[MamboToolbox::IA,"MakeDescription"];Abort[]),
 		(AppendTo[localev,"Type"->"Description"];AppendTo[localev,"Size"->{rows,cols}];
 		Do[Do[AppendTo[localev,{i,j}->ev[[cols*(i-1)+j]]],{i,rows}],{j,cols}];Return[localev])]
]

NullDescription[rows_,cols_] := MakeDescription[Table[NullVector,{i,rows*cols}],rows]  
VelocityDescription[ev_] := If[Apply[And,Map[MamboVectorQ,ev]],MakeDescription[ev,Length[ev]],(Message[MamboToolbox::IA,"VelocityDescription"];Abort[])]  
MomentumDescription[ev_] := If[Apply[And,Map[MamboVectorQ,ev]],MakeDescription[ev,1],(Message[MamboToolbox::IA,"MomentumDescription"];Abort[])]  
ForceDescription[ev_] := If[Apply[And,Map[MamboVectorQ,ev]],MakeDescription[ev,1],(Message[MamboToolbox::IA,"ForceDescription"];Abort[])]  

ClearZero[ev_] := Module[{localev,i,j},
 	If[MamboVectorQ[ev],
 		(localev = NullVector;
 		Do[If[Not[SameQ[Norm["Coordinates"/.(i/.ev),1],0]],
 			(localev=Append[Complement[localev,Cases[localev,x:("Size"->_)]],"Size"->("Size"/.localev)+1];
 			AppendTo[localev,("Size"/.localev)->(i/.ev)])],{i,"Size"/.ev}];
 			Return[localev]), 
		If[MamboDescriptionQ[ev],
 			(localev = ev;Do[Do[localev=Append[Complement[localev,Cases[localev,x:({i,j}->_)]],{i,j}->ClearZero[{i,j}/.localev]], 
				{j,("Size"/.ev)[[2]]}],{i,("Size"/.ev)[[1]]}];Return[localev]), 
			(Message[MamboToolbox::IA,"ClearZero"];Abort[])]]
] 	 

MakeTranslations[ev__] := If[Apply[And,Map[ListQ,{ev}]],Fold[MamboPlus,NullVector,Map[MakeTranslation,{ev}]],Return[MakeTranslations[{ev}]]]  
MakeTranslation[args__] := Module[{direction={0,0,0}},
 	If[args=={},Return[NullVector]];
 	If[RuleQ[args] && MamboVectorQ[args],Return[args]];
 	If[Length[args]>1 && Not[ListQ[args[[1]]/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,args];Abort[])];
 	If[Length[args]==2 && IntegerQ[args[[2]]],direction[[args[[2]]]]=1,
 		If[Length[args]==4,direction=Take[args,{2,4}],(Message[MamboToolbox::IA,"MakeTranslation"];Abort[])]];
 	Return[ClearZero[{"Type"->"Vector" ,"Size"->1,1->{"Coordinates"->direction,"Triad"->args[[1]]}}]]
]  

MamboPlus[ev1_?(MamboVectorQ[#] || MamboDescriptionQ[#]&),ev2_?(MamboVectorQ[#] || MamboDescriptionQ[#]&)] := Module[{localev=ev1,i,j=1},
 	If[Not[Equal["Type"/.ev1,"Type"/.ev2]],(Message[MamboToolbox::IA,"MamboPlus"];Abort[]),
 		(If[MamboDescriptionQ[ev1],
 			If[Not[Equal["Size"/.ev1,"Size"/.ev2]],(Message[MamboToolbox::IA,"MamboPlus"];Abort[]),
 				Do[Do[localev=Append[Complement[localev,Cases[localev,x:({i,j}->_)]],{i,j}->MamboPlus[{i,j}/.ev1,{i,j}/.ev2]],
 					{i,("Size"/.ev1)[[1]]}],{j,("Size"/.ev1)[[2]]}]],
 			If[MamboVectorQ[ev1],
 				Do[(While[j<=("Size"/.ev1) && Not[SameQ["Triad"/.(i/.ev2),"Triad"/.(j/.ev1)]],j++];
 					If[j<=("Size"/.ev1),localev=Append[Complement[localev,Cases[localev,x:(j->_)]],
 						j->{"Coordinates"->Plus["Coordinates"/.(j/.ev1),"Coordinates"/. (i/.ev2)],"Triad"->("Triad"/.(j/.ev1))}],
 						(localev=Append[Complement[localev,Cases[localev,x:("Size"->_)]],"Size"->("Size"/.localev)+1]; 
						AppendTo[localev,("Size"/.localev)->(i/.ev2)])]),{i,"Size"/.ev2}]]];
 		Return[ClearZero[localev]])]
] 			 

MamboTimes[scal_,ev_?(MamboVectorQ[#] || MamboDescriptionQ[#]&)] := Module[{localev=ev,i,j},
 	If[MamboDescriptionQ[ev],
 		Do[Do[localev=ReplacePart[localev,{i,j}->MamboTimes[scal,{i,j}/.ev],Position[localev,{i,j}->_]],
 			{j,("Size"/.ev)[[2]]}],{i,("Size"/.ev)[[1]]}],
 			Do[localev=ReplacePart[localev,i->{"Coordinates"->Times[scal,"Coordinates"/.(i/.ev)],"Triad"->("Triad"/.(i/.ev))},Position[localev,i->_]],{i,"Size"/.ev}]];
 	Return[ClearZero[localev]]
]

MamboMinus[ev1_?(MamboVectorQ[#] || MamboDescriptionQ[#]&),ev2_?(MamboVectorQ[#] || MamboDescriptionQ[#]&)] := MamboPlus[ev1,MamboTimes[-1,ev2]]
MamboMinus[ev_?(MamboVectorQ[#] || MamboDescriptionQ[#]&)] := MamboTimes[-1,ev]  
MakeRotations[rotmat__] := If[Apply[And,Map[ListQ,{rotmat}]],Fold[Dot,IdentityMatrix[3],Map[MakeRotation,{rotmat}]],Return[MakeRotations[{rotmat}]]]  

MakeRotation[args__] := Module[{lambda={0,0,0},i,j,k},
 	If[args=={},Return[IdentityMatrix[3]]];
 	If[MatrixQ[args] && Dimensions[args]=={3,3},Return[args]];
 	If[Length[args]==2 && Not[ListQ[args[[1]]]] && IntegerQ[args[[2]]],lambda[[args[[2]]]]=1,
 		If[Length[args]==4 && Apply[And,Map[Not[ListQ[#]]&,args]],
 			lambda=Take[args,{2,4}]/Sqrt[Dot[Take[args,{2,4}],Take[args,{2,4}]]],(Message[MamboToolbox::IA,MakeRotations];Abort[])]];
 	Return[Table[KroneckerDelta[i,j]*Cos[args[[1]]]+(1-Cos[args[[1]]])*lambda[[i]]*lambda[[j]]-Sin[args[[1]]]*Sum[Signature[{i,j,k}]*lambda[[k]],{k,3}],{i,3},{j,3}]]
]

MamboDot[ev1_?(MamboVectorQ[#] || MamboDescriptionQ[#]&),ev2_?(MamboVectorQ[#] || MamboDescriptionQ[#]&)]  := Module[{result,i,j,k,rotation},
 	If[Not[Equal["Type"/.ev1,"Type"/.ev2]],(Message[MamboToolbox::IA,"MamboDot1"];Abort[]), 
		(If[MamboDescriptionQ[ev1],
 			If[Not[Equal[("Size"/.ev1)[[2]],("Size"/.ev2)[[1]]]],(Message[MamboToolbox::IA,"MamboDot2"];Abort[]),
 				(result=Table[0,{i,("Size"/.ev1)[[1]]},{j,("Size"/.ev2)[[2]]}];
 				Do[Do[Do[AddTo[result[[i,j]],MamboDot[{i,k}/.ev1,{k,j}/.ev2]],{k,("Size"/.ev1)[[2]]}],{j,("Size"/.ev2)[[2]]}],{i,("Size"/.ev1)[[1]]}])], 
			If[MamboVectorQ[ev1], 
				(result=0; 
					Do[Do[(rotation=FindRotation["Triad"/.(i/.ev1),"Triad"/.(j/.ev2)];
 						AddTo[result,Dot[Replace["Coordinates",i/.ev1],Dot[rotation,"Coordinates"/.(j/.ev2)]]]),{j,"Size"/.ev2}],{i,"Size"/.ev1}])]];
 		Return[result])];
]

MamboNorm[ev_?MamboVectorQ] := Sqrt[MamboDot[ev,ev]];
MamboCross[ev1_?MamboVectorQ,ev2_?MamboVectorQ] := Module[{result=NullVector,i,j,rotation},
 	Do[Do[(rotation=FindRotation["Triad"/.(i/.ev1),"Triad"/.(j/.ev2)]; 
		result=MamboPlus[result,{"Type"->"Vector","Size"->1,
 			1->{"Coordinates"->Cross["Coordinates"/.(i/.ev1),Dot[rotation,"Coordinates"/.(j/.ev2)]],"Triad"->("Triad"/.(i/.ev1))}}];),{j,"Size"/.ev2}],{i,"Size"/.ev1}];
 	Return[ClearZero[result]];
]

MamboExpress[ev_?MamboVectorQ,triad_] := Module[{result={0,0,0},i},
 	Do[AddTo[result,Dot[FindRotation[triad,"Triad"/.(i/.ev)],"Coordinates"/.(i/.ev)]],{i,"Size"/.ev}];
 	Return[ClearZero[{"Type"->"Vector","Size"->1,1->{"Coordinates"->result,"Triad"->triad}}]];
]  

MamboSimplify[ev_?MamboVectorQ] := Return[ClearZero[Simplify[ev]]];  
DeclareStates[args__] := If[Apply[And,Map[NameQ[ToString[#]]&,{args}]],GlobalStates=Union[GlobalStates,{args}];,(Message[MamboToolbox::IA,"DeclareStates"];Abort[])];
  (* Note use of vars=Table[Symbol[ToString[#]<>ToString[i]],{i,20}]&[q] to generate sequence and then Apply[DeclareStates,vars] to process entire list. *)

DefineObjects[args__] := Module[{object,properties},
 	Backup;
 	If[Apply[And,Map[ListQ,{args}]],
 		Map[(
 			If[Not[ListQ[#[[1]]/.GlobalObserverDefinitions]],(Message[DeclareName::MT4,#[[1]]];Undo;Abort[])]; 
			If[Not[MemberQ[{"Sphere","Cylinder","Block"},#[[2]]]],(Message[MamboToolbox::IA,"DefineObjects"];Undo;Abort[])]; 
			object={}; 
			AppendTo[object,"Type"->#[[2]]];
 			Do[Which[ 	
				MamboVectorQ["point"/.#[[i]]],AppendTo[object,"Point"->"point"/.#[[i]]], 
					NameQ[ToString["point"/.#[[i]]]] && ListQ[("point"/.#[[i]])/.GlobalPointDeclarations], 
						AppendTo[object,"Point"->FindTranslation["Point"/.(#[[1]]/.GlobalObserverDefinitions),"point"/.#[[i]]]],
 					MatrixQ["orient"/.#[[i]]] && Dimensions["orient"/.#[[i]]]=={3,3},AppendTo[object,"Orient"->"orient"/.#[[i]]],
 					NameQ[ToString["orient"/.#[[i]]]] && ListQ[("orient"/.#[[i]])/.GlobalTriadDeclarations], 
						AppendTo[object,"Orient"->FindRotation["Triad"/.(#[[1]]/.GlobalObserverDefinitions),"orient"/.#[[i]]]],
 					MatchQ[#[[i]],"radius"->_] && MemberQ[{"Sphere","Cylinder"},"Type"/.object],AppendTo[object,"Radius"->"radius"/.#[[i]]],
 					MatchQ[#[[i]],"length"->_] && MemberQ[{"Cylinder"},"Type"/.object],AppendTo[object,"Length"->"length"/.#[[i]]], 
					MatchQ[#[[i]],"xlength"->_] && MemberQ[{"Block"},"Type"/.object],AppendTo[object,"XLength"->"xlength"/.#[[i]]], 
					MatchQ[#[[i]],"ylength"->_] && MemberQ[{"Block"},"Type"/.object],AppendTo[object,"YLength"->"ylength"/.#[[i]]],
 					MatchQ[#[[i]],"zlength"->_] && MemberQ[{"Block"},"Type"/.object],AppendTo[object,"ZLength"->"zlength"/.#[[i]]],
 					MatchQ[#[[i]],"color"->_] && StringQ["color"/.#[[i]]] && MemberQ[{"red","green","blue","yellow","white","black","magenta","cyan"},
 						"color"/.#[[i]]],AppendTo[object,"Color"->"color"/.#[[i]]], 
					MatchQ[#[[i]],"color"->_] && StringQ["color"/.#[[i]]] && ListQ[ToExpression["color"/.#[[i]]]] && Dimensions[ToExpression["color"/.#[[i]]]]=={3}, 
						AppendTo[object,"Color"->"color"/.#[[i]]], 
					True,(Message[MamboToolbox::IA,"DefineObjects"];Undo;Abort[]) 
				],
 			{i,3,Length[#]}]; 
			GlobalObjectDefinitions=ReplacePart[GlobalObjectDefinitions,#[[1]]->Append[#[[1]]/.GlobalObjectDefinitions,object],Position[GlobalObjectDefinitions,#[[1]]->_]];
 		)&,{args}],
 		DefineObjects[{args}] 
	];
]  

SetTime[time_] := (t=time;) 

DiffTime[args_] := If[ListQ[args],Map[DiffTime,args],
 	Append[Table[Derivative[1][GlobalStates[[i]]],{i,Length[GlobalStates]}],1].Append[Table[D[args,GlobalStates[[i]]],{i,Length[GlobalStates]}],D[args,t]]]; 
DiffTime[ev_?((MamboVectorQ[#] || MamboDescriptionQ[#])&),triad_] := Module[{result,i,j,temp},
 	If[MamboVectorQ[ev],
 		(result=NullVector;
 		Do[(temp={"Type"->"Vector","Size"->1,1->{"Coordinates"->("Coordinates"/.(i/.ev)),"Triad"->("Triad"/.(i/.ev))}}; 
			result=MamboPlus[MamboPlus[result,{"Type"->"Vector","Size"->1,1->{"Coordinates"->DiffTime["Coordinates"/.(i/.ev)],"Triad"->("Triad"/.(i/.ev))}}],
 				MamboCross[AngularVelocity[triad,"Triad"/.(i/.ev)],temp]];)
 		,{i,"Size"/.ev}];),
 		(result=ev;
 		Do[Do[result=ReplacePart[result,{i,j}->DiffTime[{i,j}/.result,triad],Position[result,{i,j}->_]],{i,("Size"/.ev)[[1]]}],{j,("Size"/.ev)[[2]]}]); 
	];
 	Return[result];
] 

AngularVelocity[triad1_,triad2_] := Module[{path,conn,rotation,angvel},
 	If[Not[NameQ[ToString[triad1]]] || Not[NameQ[ToString[triad2]]],(Message[MamboToolbox::IA,"AngularVelocity"];Abort[])];
 	If[Not[ListQ[triad1/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,triad1];Abort[])];
 	If[Not[ListQ[triad2/.GlobalTriadDeclarations]],(Message[DeclareName::MT2,triad2];Abort[])];
 	If[SameQ[triad1,triad2],Return[NullVector]];
 	If[MamboVectorQ[{triad1,triad2}/.GlobalAngularVelocity],Return[{triad1,triad2}/.GlobalAngularVelocity]];
 	path=MamboFindPath[triad1,triad2,GlobalTriadDeclarations];
 	If[path=={},(Message[DeclareName::MT7,triad1,triad2];Abort[])]; 
	If[Length[path]==2,
 		(rotation=FindRotation[triad1,triad2]; 
		angvel=Transpose[rotation].DiffTime[rotation]; 
		angvel=Simplify[MakeTranslations[triad2,angvel[[3,2]],angvel[[1,3]],angvel[[2,1]]]];
 		GlobalAngularVelocity=Union[GlobalAngularVelocity,{{triad1,triad2}->angvel,{triad2,triad1}->MamboMinus[angvel]}];),
 		Do[If[Not[MamboVectorQ[{triad1,path[[conn]]}/.GlobalAngularVelocity]],
 			(GlobalAngularVelocity=Union[GlobalAngularVelocity, 
				{{triad1,path[[conn]]}->MamboPlus[AngularVelocity[triad1,path[[conn-1]]],AngularVelocity[path[[conn-1]],path[[conn]]]]}];
 			GlobalAngularVelocity=Union[GlobalAngularVelocity, 
				{{path[[conn]],triad1}->MamboMinus[{triad1,path[[conn]]}/.GlobalAngularVelocity]}])], 
		{conn,2,Length[path]}]
 	];
 	Return[{triad1,triad2}/.GlobalAngularVelocity]; 
];		 

LinearVelocity[observer_,point_] := (
 	If[Not[NameQ[ToString[observer]] && NameQ[ToString[point]]],(Message[MamboToolbox::IA,"LinearVelocity"];Abort[])];
 	If[Not[ListQ[observer/.GlobalObserverDefinitions]],(Message[DeclareName::MT3,observer];Abort[])];
 	Return[DiffTime[FindTranslation["Point"/.(observer/.GlobalObserverDefinitions),point],"Triad"/.(observer/.GlobalObserverDefinitions)]]; 
) 

LinearMomentum[observer_,point_,mass_] := MamboTimes[mass,LinearVelocity[observer,point]]; 
LinearMomentum[observer_,point_] := LinearMomentum[observer,point,Global`M]; 
AngularMomentum[observer_,triad_,mass_?((MatrixQ[#] && Dimensions[#]=={3,3})&)] := Module[{localev,i},
 	If[Not[ListQ[observer/.GlobalObserverDefinitions]],(Message[MamboToolbox::IA,"AngularMomentum"];Abort[])];
 	localev=AngularVelocity["Triad"/.(observer/.GlobalObserverDefinitions),triad];
 	Do[localev=ReplacePart[localev,
 		i->{"Coordinates"->Dot[Dot[FindRotation["Triad"/.(i/.localev),triad],Dot[mass,FindRotation[triad,"Triad"/.(i/.localev)]]],"Coordinates"/.(i/.localev)],"Triad"->("Triad"/.(i/.localev))},
 		Position[localev,i->_]],{i,"Size"/.localev}];
 	Return[localev]; 
] 
AngularMomentum[observer_,triad_] := Module[{mass},
 	Off[General::spell1];
 	mass=Table[Symbol["Global`Inertia"<>ToString[i]<>ToString[j]],{i,3},{j,3}];
 	On[General::spell1];
 	Return[AngularMomentum[observer,triad,mass]]; 
]  

CoeffExtract[ev_?MamboDescriptionQ,coeffs_?ListQ] := Module[{localev,betamatrix=NullDescription[("Size"/.ev)[[1]],Length[coeffs]],i,j,k,result,temp},
 	If[("Size"/.ev)[[2]]!=1,(Message[MamboToolbox::IA,"CoeffExtract"];Abort[])];
 	Do[Do[(
 		localev={i,1}/.ev; 
		result=NullVector; 
		Do[(temp=CoefficientArrays["Coordinates"/.(k/.localev),coeffs[[j]]];
 			localev=ReplacePart[localev,k->{"Coordinates"->If[Length[temp]==2,Flatten[Normal[temp[[2]]]],{0,0,0}],"Triad"->("Triad"/.(k/.localev))},
 				Position[localev,k->_]];
 			result=MamboPlus[result,{"Type"->"Vector","Size"->1,1->(k/.localev)}];),
 		{k,"Size"/.localev}];
 		betamatrix=ReplacePart[betamatrix,{i,j}->result,Position[betamatrix,{i,j}->_]];),
 	{j,Length[coeffs]}],{i,("Size"/.ev)[[1]]}]; 
	Return[betamatrix]; 
]  

PointOut[observer1_,observer2_] := Module[{translation,i},
 	If[ListQ[observer1] || ListQ[observer2],(Message[MamboToolbox::IA,"PointOut"];Abort[])];
 	If[Not[ListQ[observer1/.GlobalObserverDefinitions]] || Not[ListQ[observer2/.GlobalObserverDefinitions]],Return["Null"]];
 	If[MamboFindPath["Point"/.(observer1/.GlobalObserverDefinitions),"Point"/.(observer2/.GlobalObserverDefinitions),GlobalPointDeclarations]=={},Return["Empty"]];
 	translation=FindTranslation["Point"/.(observer1/.GlobalObserverDefinitions),"Point"/.(observer2/.GlobalObserverDefinitions)];
 	If[("Size"/.translation)==0,Return[{0,0,0}]];
 	Do[If[MamboFindPath["Triad"/.(observer1/.GlobalObserverDefinitions),"Triad"/.(i/.translation),GlobalTriadDeclarations]=={},Return["Empty"]],{i,"Size"/.translation}];
 	Return[Table[MamboDot[translation,MakeTranslations["Triad"/.(observer1/.GlobalObserverDefinitions),i]],{i,3}]]; 
] 

OrientOut[observer1_,observer2_] := Module[{rotation,i,j},
 	If[ListQ[observer1] || ListQ[observer2],(Message[MamboToolbox::IA,"OrientOut"];Abort[])];
 	If[Not[ListQ[observer1/.GlobalObserverDefinitions]] || Not[ListQ[observer2/.GlobalObserverDefinitions]],Return["Null"]];
 	If[MamboFindPath["Triad"/.(observer1/.GlobalObserverDefinitions),"Triad"/.(observer2/.GlobalObserverDefinitions),GlobalTriadDeclarations]=={},Return["Empty"]];
 	rotation=FindRotation["Triad"/.(observer1/.GlobalObserverDefinitions),"Triad"/.(observer2/.GlobalObserverDefinitions)];
 	Return[Flatten[Table[rotation[[i,j]],{i,3},{j,3}]]];  
] 

CheckArgumentsRaw[f_,x_] := Module[{match=False,i=1},
 	If[ListQ[x],
 		(If[ListQ[f],Map[CheckArgumentsRaw[#,x]&,f]];
 		If[NumericQ[f],Return[True]];
 		If[NameQ[ToString[f]], 
			(While[!match && i<=Length[x],(match=SameQ[f,x[[i]]];i++)];
 			If[match,Return[match],(Message[DeclareName::MT9,f];Abort[])];),
 			Map[CheckArgumentsRaw[#,x]&,Apply[List,f]]];),
 		CheckArgumentsRaw[f,{x}]];
 	Return[True]; 
] 

CheckArguments[f_,x_] := Module[{match=False,i=1},
 	If[ListQ[x],
 		(If[ListQ[f],Return[And@@Map[CheckArguments[#,x]&,f]]];
 		If[NumericQ[f],Return[True]];
 		If[NameQ[ToString[f]],
 			(While[!match && i<=Length[x],(match=SameQ[f,x[[i]]];i++)]; 
			Return[match];), 
			Return[And@@Map[CheckArguments[#,x]&,Apply[List,f]]]];), 
		Return[CheckArguments[f,{x}]]]; 
	Return[match]; 
] 

PowerRewrite[string_?StringQ] := Module[{temp=string,powis=StringPosition[string,"pow(", 1],
 						startpos,middlepos,endpos,found=False,arg1,arg2},
 	While[Length[powis] > 0,
 		arg1=.;
 		arg2=.;
 		startpos = powis[[1, 1]];
 		middlepos = powis[[1, 2]];
 		endpos = middlepos+StringPosition[StringTake[temp,{middlepos+1,StringLength[temp]}],",",1][[1,1]];
 		found = False;
 		While[Not[found],
 			found = Equal[Length[StringPosition[StringTake[temp,{middlepos+1,endpos-1}],"("]]
 				-Length[StringPosition[StringTake[temp,{middlepos+1,endpos-1}],")"]],0];
 			If[found,arg1 = StringTake[temp,{middlepos+1,endpos-1}],
 				endpos = endpos+StringPosition[StringTake[temp,{endpos+1,StringLength[temp]}],",",1][[1,1]]]
 		];
 		middlepos = endpos; 
		endpos = middlepos+StringPosition[StringTake[temp,{middlepos+1,StringLength[temp]}],")",1][[1,1]];
 		found = False;
 		While[Not[found], 
			found = Equal[Length[StringPosition[StringTake[temp,{middlepos+1,endpos-1}],"("]]
 				-Length[StringPosition[StringTake[temp,{middlepos+1,endpos-1}],")"]],0]; 
			If[found,arg2 = StringTake[temp,{middlepos+1,endpos-1}],
 				endpos = endpos+StringPosition[StringTake[temp,{endpos+1,StringLength[temp]}],")",1][[1,1]]]
 		];
 		temp = StringReplacePart[temp,"("<>arg1<>")^("<>arg2<>")",{startpos,endpos}];
 		powis = StringPosition[temp,"pow(",1];
 	];
 	Return[temp]; 
]  

GeometryOutput[args__] := Module[{i,temp={args},options={"indent"->0}},
 	If[Position[temp,"main"->_]=={},(Message[GeometryOutput::GO1];Abort[])];
 	If[Position[temp,"checkargs"]!={},(AppendTo[options,"checkargs"->True];temp=Complement[temp,{"checkargs"}]),AppendTo[options,"checkargs"->False]];
 	If[Position[temp,"checktree"]!={},(AppendTo[options,"checktree"->True];temp=Complement[temp,{"checktree"}]),AppendTo[options,"checktree"->False]];
 	If[Position[temp,"states"->_]=={},AppendTo[options,"states"->{}]];
 	If[Position[temp,"parameters"->_]=={},AppendTo[options,"parameters"->{}]];
 	If[Position[temp,"anims"->_]=={},AppendTo[options,"anims"->{}]];
 	If[Not[RuleQ[temp]],(Message[MamboToolbox::IA,"GeometryOutput1"];Abort[])];
 	Do[Which[
 		NameQ[ToString["main"/.temp[[i]]]] && Not[SameQ["main","main"/.temp[[i]]]],(If[Not[ListQ[("main"/.temp[[i]])/.GlobalObserverDeclarations]],(Message[DeclareName::MT2,"main"/.options];Abort[]),
 			AppendTo[options,"main"->("main"/.temp[[i]])]];AppendTo[options,"root"->("main"/.options)]),
 		ListQ["states"/.temp[[i]]] && Apply[And,Map[NameQ[ToString[#]] || SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && NumberQ[Part[#,2]]& ,"states"/.temp[[i]]]],
 			AppendTo[options,"states"->Map[If[NameQ[ToString[#]],#,Part[#,1]]&,"states"/.temp[[i]]]],
 		ListQ["parameters"/.temp[[i]]] && Apply[And,Map[NameQ[ToString[#]] || SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && NumberQ[Part[#,2]]& ,"parameters"/.temp[[i]]]], 
			AppendTo[options,"parameters"->Map[If[NameQ[ToString[#]],#,Part[#,1]]&,"parameters"/.temp[[i]]]],
 		ListQ["anims"/.temp[[i]]] && Apply[And,Map[NameQ[ToString[#]] || SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && Not[ListQ[Part[#,2]]]& ,"anims"/.temp[[i]]]],
 			AppendTo[options,"anims"->Map[If[NameQ[ToString[#]],#,Part[#,1]]&,"anims"/.temp[[i]]]],
 		StringQ["filename"/.temp[[i]]] && Not[SameQ["filename","filename"/.temp[[i]]]],AppendTo[options,"filename"->("filename"/.temp[[i]])],
 		True,(Message[MamboToolbox::IA,"GeometryOutput2"];Abort[])],
 	{i,Length[temp]}];
 	If[Position[options,"filename"->_]!={} && Not[SameQ["filename"/.options,"stdout"]],
 		(AppendTo[options,"stream"->OpenWrite["filename"/.options,FormatType->OutputForm,PageWidth->Infinity]];GeomOut[options];Close["stream"/.options]), 
		(AppendTo[options,"stream"->"stdout"];GeomOut[options])]; 
]  

GeomOut[options_?RuleQ] := Module[{i,localoptions,stream="stream"/.options,main="main"/.options,root="root"/.options,indent="indent"/.options,
 					args=Union["states"/.options,"parameters"/.options,"anims"/.options,{t}],position,orientation,temp},
 	If[SameQ[main,root],
 		(Write[stream,"MODULE "<>ToString[main]<>" {"];
 		Map[(localoptions=ReplacePart[options,"indent"->indent+1,Position[options,"indent"->_]];
 			ObjectOutput[localoptions,#])&,main/.GlobalObjectDefinitions];
 		Map[(localoptions=ReplacePart[options,"indent"->indent+1,Position[options,"indent"->_]];
 			localoptions=ReplacePart[localoptions,"main"->#,Position[localoptions,"main"->_]];
 			GeomOut[localoptions])&,main/.GlobalObserverDeclarations]),
  		(Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent}],"BODY "<>ToString[main]<>" {"]]];
 		position=PointOut[root,main];
 		If[Not[SameQ[position,"Null"]],
 			If[SameQ[position,"Empty"],(
 				Write[stream,StringJoin[Table["   ",{i,indent+1}],"Point {}"]];
 				If["checktree"/.options,(Close[stream];DeleteFile["filename"/.options];Message[GeometryOutput::GO2,"GeomOut1"];Abort[])];
	(*No path between reference points of observers`,args[1],args[2]); *)
 			),( 
				If["checkargs"/.options, If[Not[CheckArguments[position,args]], 
					(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,position];Abort[])]];
 				temp=Map[ToString[Format`CAssign[#,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]&,position];
 				Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Point {",temp[[1]],",",temp[[2]],",",temp[[3]],"}"]]];
 			)
 			];
 		];
 		orientation=OrientOut[root,main];
 		If[Not[SameQ[orientation,"Null"]],
 			If[SameQ[orientation,"Empty"], 
				(Write[stream,StringJoin[Table["   ",{i,indent+1}],"Orient {}"]];
 				If["checktree"/.options,(Close[stream];DeleteFile["filename"/.options];Message[GeometryOutput::GO2,"GeomOut2"];Abort[])];(*No path between reference triads of observers`,args[1],args[2]); *)
 			),(
 				If["checkargs"/.options, If[Not[CheckArguments[orientation,args]],
 					(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,orientation];Abort[])]];
 				temp=Map[ToString[Format`CAssign[#,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]&,Flatten[orientation]];
 				Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Orient {",Table[temp[[i]]<>",",{i,8}],temp[[9]],"}"]]];
 			)
 			];
 		];
 		Map[(localoptions=ReplacePart[options,"indent"->indent+1,Position[options,"indent"->_]];
 			ObjectOutput[localoptions,#])&,main/.GlobalObjectDefinitions];
 		Map[(localoptions=ReplacePart[options,"indent"->indent+1,Position[options,"indent"->_]];
 			localoptions=ReplacePart[localoptions,"main"->#,Position[localoptions,"main"->_]];
 			localoptions=ReplacePart[localoptions,"root"->main,Position[localoptions,"root"->_]];
 			GeomOut[localoptions])&,Complement[main/.GlobalObserverDeclarations,{root}]];)
 	];
 	Write[stream,StringJoin[Table["   ",{i,indent}],"}"]]; 
]  

ObjectOutput[options_?RuleQ,object_?RuleQ] := Module[{i=1,temp,main="main"/.options,stream="stream"/.options,indent="indent"/.options,nopath=False,
 							args=Union["states"/.options,"parameters"/.options,"anims"/.options,{t}]},
 	Write[stream,StringJoin[Table["   ",{i,indent}],"Type"/.object," {"]];
 	If[Position[object,"Point"->_]!={},
 		If[Equal["Size"/.("Point"/.object),0],Write[stream,StringJoin[Table["   ",{i,indent+1}],"Point {0,0,0}"]],
 			(While[!nopath && i<="Size"/.("Point"/.object),
 				(Print[i];If[MamboFindPath["Triad"/.(main/.GlobalObserverDefinitions),"Triad"/.(i/.("Point"/.object)),GlobalTriadDeclarations]=={},nopath=True];i++)];
 			If[nopath,(Write[stream,StringJoin[Table["   ",{i,indent+1}],"Point {}"]];
				  If["checktree"/.options,(Close[stream];DeleteFile["filename"/.options];Message[GeometryOutput::GO2,"ObjectOutput1"];Abort[])]),
   (*No path from reference triad of parent observer to object's point vector!*)
 				(temp="Coordinates"/.(1/.MamboExpress["Point"/.object,"Triad"/.(main/.GlobalObserverDefinitions)]);
 				If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 					(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 				temp=Map[ToString[Format`CAssign[#,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]&,temp];
 				Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Point {",temp[[1]],",",temp[[2]],",",temp[[3]],"}"]]]; 
				)
 			])
 		]
 	];
 	If[Position[object,"Orient"->_]!={},
 		(temp=Flatten["Orient"/.object];
 		If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		temp=Map[ToString[Format`CAssign[#,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]&,temp];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Orient {",Table[temp[[i]]<>",",{i,8}],temp[[9]],"}"]]]; 
		)
 	];
 	If[Position[object,"Radius"->_]!={},
 		(temp="Radius"/.object;
 		If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Radius ",ToString[Format`CAssign[temp,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]]]];
 		)
 	];
 	If[Position[object,"Length"->_]!={},
 		(temp="Length"/.object;
 		If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Length ",ToString[Format`CAssign[temp,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]]]];
 		)
 	];
 	If[Position[object,"XLength"->_]!={},
 		(temp="XLength"/.object; 
		If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Xlength ",ToString[Format`CAssign[temp,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]]]];
 		)
 	];
 	If[Position[object,"YLength"->_]!={},
 		(temp="YLength"/.object; 
		If["checkargs"/.options,If[Not[CheckArguments[temp,args]], 
			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Ylength ",ToString[Format`CAssign[temp,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]]]]; 
		)
 	];
 	If[Position[object,"ZLength"->_]!={}, 
		(temp="ZLength"/.object;
 		If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Zlength ",ToString[Format`CAssign[temp,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]]]];
 		)
 	];
 	If[Position[object,"Color"->_]!={},
 		(temp=Switch["Color"/.object,
 			"red",{1,0,0},
 			"green",{0,1,0},
 			"blue",{0,0,1}, 
			"yellow",{1,1,0},
 			"magenta",{1,0,1},
 			"cyan",{0,1,1},
 			"white",{1,1,1}, 
			"black",{0,0,0},
 			_,ToExpression["Color"/.object]];
 		If["checkargs"/.options,If[Not[CheckArguments[temp,args]],
 			(Close[stream];DeleteFile["filename"/.options];Message[DeclareName::MT9,temp];Abort[])]];
 		temp=Map[ToString[Format`CAssign[#,Format`AssignBreak->{4000,"\n"},Format`AssignEnd->"",Format`AssignOptimize->False]]&,temp];
 		Write[stream,PowerRewrite[StringJoin[Table["   ",{i,indent+1}],"Color {",temp[[1]],",",temp[[2]],",",temp[[3]],"}"]]];
 		)
 	];
 	Write[stream,StringJoin[Table["   ",{i,indent}],"}"]]; 
]		   

MotionOutput[args__] := Module[{i,j,A,lhs,rhs,eqs,breakpoint=0,temp={args},checkargs=False,checksings=False,
 				ode={},states={},parameters={},anims={},insignals={},output="stdout",vars={},insigs,anis,eqs1},
 	If[Position[temp,"checkargs"]!={},(checkargs=True;temp=Complement[temp,{"checkargs"}])];
 	If[Position[temp,"checksings"]!={},(checksings=True;temp=Complement[temp,{"checksings"}])];
 	If[Not[RuleQ[temp]],(Message[MamboToolbox::IA,"MotionOutput1"];Abort[])];
 	Do[Which[
 		ListQ["states"/.temp[[i]]] && Apply[And,Map[NameQ[ToString[#]] || SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && NumberQ[Part[#,2]]& ,"states"/.temp[[i]]]], 
			(PrependTo[vars,Map[If[NameQ[ToString[#]],#,Part[#,1]]&,"states"/.temp[[i]]]];states=Map[If[NameQ[ToString[#]],#->0,#]&,"states"/.temp[[i]]]),
 		ListQ["parameters"/.temp[[i]]] && Apply[And,Map[NameQ[ToString[#]] || SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && NumberQ[Part[#,2]]& ,"parameters"/.temp[[i]]]],
 			(AppendTo[vars,Map[If[NameQ[ToString[#]],#,Part[#,1]]&,"parameters"/.temp[[i]]]];parameters=Map[If[NameQ[ToString[#]],#->0,#]&,"parameters"/.temp[[i]]]),
 		ListQ["anims"/.temp[[i]]] && Apply[And,Map[SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && Not[ListQ[Part[#,2]]]& ,"anims"/.temp[[i]]]], 
			(AppendTo[vars,Map[Part[#,1]&,"anims"/.temp[[i]]]];anims="anims"/.temp[[i]]), 
		ListQ["insignals"/.temp[[i]]] && Apply[And,Map[SameQ[Head[#],Rule] && NameQ[ToString[Part[#,1]]] && Not[ListQ[Part[#,2]]]& ,"insignals"/.temp[[i]]]],
 			(AppendTo[vars,Map[Part[#,1]&,"insignals"/.temp[[i]]]];insignals="insignals"/.temp[[i]]),
 		ListQ["ode"/.temp[[i]]],ode="ode"/.temp[[i]],
 		StringQ["filename"/.temp[[i]]] && Not[SameQ["filename","filename"/.temp[[i]]]],output="filename"/.temp[[i]],
 		True,(Message[MamboToolbox::IA,"MotionOutput2"];Abort[])],
 	{i,Length[temp]}];
 	vars=Flatten[Append[vars,t]]; 
	If[checkargs,(CheckArgumentsRaw[ode,vars];CheckArgumentsRaw[insignals,vars];CheckArgumentsRaw[anims,vars])];
 	If[ode!={},(
 		If[Length[ode]!=Length[states],(Message[MamboToolbox::NO,"MotionOutput3"];Abort[])];
 		A=CoefficientArrays[ode,Map[Derivative[1][Part[#,1]]&,states]]//Normal;
 		If[checksings,If[MatrixRank[A[[2]]/.N[insignals]/.N[states]/.N[parameters]/.t->0./.{True->1,False->0}]<Length[states],(Message[MamboToolbox::SM,"MotionOutput3"];Abort[])]]; 
		 rhs=Fold[Append,-A[[1]],Flatten[A[[2]]]];
 		lhs=Fold[Drop,Fold[Append,Table[StringJoin["rhs[",ToString[vars[[i]]],"]"],{i,Length[states]}],
 			Flatten[Table[StringJoin["mass[",ToString[vars[[i]]],"][",ToString[vars[[j]]],"]"],{i,Length[states]},{j,Length[states]}]]],Reverse[Position[rhs,0]]];
 		rhs=Fold[Drop,rhs,Reverse[Position[rhs,0]]];
 		eqs=Format`CAssign[Evaluate[lhs],rhs,Format`AssignBreak->{500,"\n     "}][[1,1]];
 		While[Not[StringMatchQ[eqs[[1+breakpoint]],"rhs*"]],breakpoint++];
 	)];
     If[Not[SameQ[output,"stdout"]],stream=OpenWrite[output,FormatType->OutputForm,PageWidth->Infinity],stream="stdout"];
 	If[Length[states]!=0,
 		Write[stream,StringJoin["states ",Map[StringJoin[ToString[Part[#,1]],"=",ToString[Part[#,2]],","]&,Take[states,{1,Length[states]-1}]],
 			StringJoin[ToString[Part[Last[states],1]],"=",ToString[Part[Last[states],2]],";"]]];
 	];
 	If[Length[parameters]!=0,
 		Write[stream,StringJoin["parameters ",Map[StringJoin[ToString[Part[#,1]],"=",ToString[Part[#,2]],","]&,Take[parameters,{1,Length[parameters]-1}]],
 			StringJoin[ToString[Part[Last[parameters],1]],"=",ToString[Part[Last[parameters],2]],";"]]];
 	];
 	Write[stream,StringJoin["time ",ToString[t],";"]];
 	If[Length[anims]!=0,(
 		Write[stream,"anims {"];
 		anis=Map[StringInsert[#,"   ",1]&,Format`CAssign[Evaluate[Map[Part[#,1]&,anims]],Map[Part[#,2]&,anims]][[1,1]]];
 		anis=StringJoin[Append[Map[StringInsert[#,"\n",-1]&,Take[anis,{1,Length[anis]-1}]],anis[[-1]]]]; 
		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compile_$\\\n     "]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compile_\\\n     $"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compile\\\n     _$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compil\\\n     e_$"]]; 
		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compi\\\n     le_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Comp\\\n     ile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Com\\\n     pile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Co\\\n     mpile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"C\\\n     ompile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compile_$\\\n"]]; 
		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compile_\\\n$"]]; 
		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compile\\\n_$"]]; 
		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compil\\\ne_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Compi\\\nle_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Comp\\\nile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Com\\\npile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"Co\\\nmpile_$"]];
 		anis=StringReplacePart[anis,"\n     anis_t",StringPosition[anis,"C\\\nompile_$"]];
 		anis=StringReplacePart[anis,"\n     ",StringPosition[anis,"\\\n"]];
 		anis=StringReplacePart[anis,"anis_t",StringPosition[anis,"Compile_$"]];
 		Write[stream,PowerRewrite[anis]];
 		Write[stream,"}\n"]
 	)];
 	If[Length[ode]!=0,(
 		eqs=Map[StringInsert[#,"   ",1]&,eqs];
 		If[Length[insignals]!=0 || breakpoint!=0,(
 			Write[stream,"insignals {"];
 			If[Length[insignals]!=0,(
 				insigs=Map[StringInsert[#,"   ",1]&,Format`CAssign[Evaluate[Map[Part[#,1]&,insignals]],Map[Part[#,2]&,insignals]][[1,1]]]; 
				insigs=StringJoin[Append[Map[StringInsert[#,"\n",-1]&,Take[insigs,{1,Length[insigs]-1}]],insigs[[-1]]]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Compile_$\n"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Compile_\n$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Compile\n_$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Compil\ne_$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Compi\nle_$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Comp\nile_$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Com\npile_$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"Co\nmpile_$"]];
 				insigs=StringReplacePart[insigs,"\nins1_t",StringPosition[insigs,"C\nompile_$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Compile_$\n     "]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Compile_\n     $"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Compile\n     _$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Compil\n     e_$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Compi\n     le_$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Comp\n     ile_$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Com\n     pile_$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"Co\n     mpile_$"]];
 				insigs=StringReplacePart[insigs,"\n     ins1_t",StringPosition[insigs,"C\n     ompile_$"]];
 				insigs=StringReplacePart[insigs,"ins1_t",StringPosition[insigs,"Compile_$"]];
				 insigs=StringReplacePart[insigs,"\n",StringPosition[insigs,"\\\n"]];
 				Write[stream,PowerRewrite[insigs]];
 			)];
 			If[breakpoint!=0,( 
				eqs1=Take[eqs,{1,breakpoint}]; 
				eqs1=StringJoin[Append[Map[StringInsert[#,"\n",-1]&,Take[eqs1,{1,Length[eqs1]-1}]],eqs1[[-1]]]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compile_$\n     "]]; 
				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compile_\n     $"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compile\n     _$"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compil\n     e_$"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compi\n     le_$"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Comp\n     ile_$"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Com\n     pile_$"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Co\n     mpile_$"]];
 				eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"C\n     ompile_$"]];
 				eqs1=StringReplacePart[eqs1,"ins2_t",StringPosition[eqs1,"Compile_$"]];
 				eqs1=StringReplacePart[eqs1," *",StringPosition[eqs1," .*"]];
 				Write[stream,PowerRewrite[eqs1]];
 			)];
 			Write[stream,"}\n"];
 		)];
 		Write[stream,"ode {"];
 		eqs1=Take[eqs,{breakpoint+1,Length[eqs]}];
 		eqs1=StringJoin[Append[Map[StringInsert[#,"\n",-1]&,Take[eqs1,{1,Length[eqs1]-1}]],eqs1[[-1]]]];
 		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compile_$\n     "]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compile_\n     $"]];
 		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compile\n     _$"]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compil\n     e_$"]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Compi\n     le_$"]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Comp\n     ile_$"]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Com\n     pile_$"]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"Co\n     mpile_$"]]; 
		eqs1=StringReplacePart[eqs1,"\n     ins2_t",StringPosition[eqs1,"C\n     ompile_$"]]; 
		eqs1=StringReplacePart[eqs1,"ins2_t",StringPosition[eqs1,"Compile_$"]];
 		eqs1=StringReplacePart[eqs1," *",StringPosition[eqs1," .*"]];
 		Write[stream,PowerRewrite[eqs1]];
 		Write[stream,"}\n"];
 	)];
 	If[Not[SameQ[output,"stdout"]],Close[stream]];
]  

End[];

EndPackage[]
