(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 10.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    147544,       4260]
NotebookOptionsPosition[    140007,       4065]
NotebookOutlinePosition[    140536,       4084]
CellTagsIndexPosition[    140493,       4081]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Mambo Toolbox commands from Multibody Mechanics and Visualization\
\>", "Chapter",
 CellChangeTimes->{{3.7038600835679197`*^9, 3.70386009994282*^9}}],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command Quit[] resets the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session memory. Alternatively, individual names may be removed from the \
session memory by assigning a period to their value. For example, to remove \
the variable temp from the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session memory, execute the command temp=.;\n\nThe Mambo Toolbox function \
Restart resets the Mambo Toolbox session memory, i.e., the Mambo Toolbox \
global variables.\n\nThe ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command Directory[] returns the full name of the current directory as a \
string. The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " command SetDirectory[\[OpenCurlyDoubleQuote]dir\[CloseCurlyDoubleQuote]] \
sets the current working directory. It may be helpful to put the Mambo \
Toolbox source files as well as your project files in a single directory and \
to use SetDirectory to move to this directory before loading the toolbox."
}], "Text",
 CellChangeTimes->{{3.70385964394722*^9, 3.703859805334793*^9}, {
  3.703860042224992*^9, 3.703860073774572*^9}, {3.703860168938833*^9, 
  3.703860169473312*^9}}],

Cell[BoxData[
 RowBox[{"Quit", "[", "]"}]], "Input"],

Cell[TextData[{
 "The following commands are necessary to load the Mambo Toolbox into ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". These commands will need to be modified to reflect the path to the \
MamboToolbox.m and Format.m files on your computer. Alternatively, open the \
files in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " and click the \[OpenCurlyDoubleQuote]Run Package\[CloseCurlyDoubleQuote] \
button."
}], "Text",
 CellChangeTimes->{{3.7038589581703978`*^9, 3.703858997676648*^9}}],

Cell[BoxData[{
 RowBox[{"Get", "[", "\"\<C:\\Mambo\\MamboToolbox.m\>\"", 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"Get", "[", "\"\<C:\\Mambo\\Format_m.txt\>\"", "]"}]}], "Input",
 CellChangeTimes->{{3.6685157368674355`*^9, 3.6685157375774565`*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 2, Section 2.5", \
"Section"]], "Text",
 CellChangeTimes->{{3.703859091984762*^9, 3.703859102985371*^9}, 
   3.703859874616503*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", \
"GlobalObserverDeclarations"}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"A", ",", "B", ",", "C"}], 
  "]"}], "\[IndentingNewLine]", "GlobalObserverDeclarations"}], "Input"],

Cell[BoxData[{
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "B"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "C"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", "GlobalObserverDeclarations"}], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "A"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "B"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "C"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"GeometryOutput", "[", 
   RowBox[{
    RowBox[{"\"\<main\>\"", "\[Rule]", "A"}], ",", 
    RowBox[{"\"\<filename\>\"", "->", "\"\<hierarchy.geo\>\""}]}], "]"}], 
  " "}]], "Input",
 CellChangeTimes->{{3.703858812251161*^9, 3.703858815014522*^9}, {
  3.7038591752190533`*^9, 3.703859175882612*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 2, Section 2.6", \
"Section"]], "Text",
 CellChangeTimes->{3.7038591857356443`*^9, 3.703859897104941*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{
  "W", ",", "Base", ",", "LowerBeam", ",", "Bracket", ",", "Shade", ",", 
   "MiddleBeam", ",", "UpperBeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Base"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "LowerBeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"LowerBeam", ",", "Bracket"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "Shade"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "MiddleBeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "UpperBeam"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], "]"}]}], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 3, Section 3.5", \
"Section"]], "Text",
 CellChangeTimes->{3.703859196183531*^9, 3.703859900673127*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{"v", "=", 
  RowBox[{"MakeTranslations", "[", 
   RowBox[{"a", ",", "1", ",", "2", ",", "0"}], "]"}]}]}], "Input"],

Cell[BoxData["v"], "Input"],

Cell[BoxData[
 RowBox[{"MakeTranslations", "[", 
  RowBox[{"a", ",", "0", ",", "0", ",", "0"}], "]"}]], "Input"],

Cell[BoxData["NullVector"], "Input"],

Cell[BoxData[{
 RowBox[{"2", "~", "MamboTimes", "~", "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboTimes", "[", 
  RowBox[{"2", ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"(", 
   RowBox[{"-", "1"}], ")"}], "~", "MamboTimes", "~", 
  "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboTimes", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"(", 
   RowBox[{"1", "/", 
    RowBox[{"MamboNorm", "[", "v", "]"}]}], ")"}], "~", "MamboTimes", "~", 
  "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboTimes", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"MamboNorm", "[", "v", "]"}]}], ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"w", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", 
     RowBox[{"-", "1"}], ",", "q", ",", "0"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"w", "~", "MamboPlus", "~", "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboPlus", "[", 
  RowBox[{"w", ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareTriads", "[", "b", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"u", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"b", ",", "r", ",", 
     RowBox[{"r", "-", "s"}], ",", "s"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"u", "~", "MamboPlus", "~", "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboPlus", "[", 
  RowBox[{"u", ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"MakeTranslations", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "1", ",", "2", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", "r", ",", 
     RowBox[{"r", "-", "s"}], ",", "s"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"(", 
   RowBox[{"v", "~", "MamboPlus", "~", "u"}], ")"}], "~", "MamboPlus", "~", 
  "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboPlus", "[", 
  RowBox[{
   RowBox[{"MamboPlus", "[", 
    RowBox[{"v", ",", "u"}], "]"}], ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"v", "~", "MamboPlus", "~", "u", "~", "MamboPlus", "~", 
  "v"}]], "Input"],

Cell[BoxData[{
 RowBox[{"v", "~", "MamboPlus", "~", "NullVector"}], "\[IndentingNewLine]", 
 RowBox[{"MamboPlus", "[", 
  RowBox[{"v", ",", "NullVector"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"v", "~", "MamboMinus", "~", "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboMinus", "[", 
  RowBox[{"v", ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"v", "~", "MamboPlus", "~", "u", "~", "MamboMinus", "~", 
  "v"}], "\[IndentingNewLine]", 
 RowBox[{"MamboMinus", "[", 
  RowBox[{
   RowBox[{"MamboPlus", "[", 
    RowBox[{"v", ",", "u"}], "]"}], ",", "v"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"v", "=."}], ";", 
   RowBox[{"w", "=."}], ";", 
   RowBox[{"u", "=."}], ";"}], " "}]], "Input",
 CellChangeTimes->{
  3.6685243022460513`*^9, {3.703858825935001*^9, 3.70385885310385*^9}, {
   3.703858919962152*^9, 3.7038589205459747`*^9}}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "v1", ",", "v2", ",", "v3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"w", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "w1", ",", "w2", ",", "w3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"v", "~", "MamboDot", "~", "w"}], "\[IndentingNewLine]", 
 RowBox[{"MamboDot", "[", 
  RowBox[{"v", ",", "w"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"v", "~", "MamboCross", "~", "w"}], "\[IndentingNewLine]", 
 RowBox[{"MamboCross", "[", 
  RowBox[{"v", ",", "w"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"v", "=."}], ";", 
   RowBox[{"w", "=."}], ";"}], " "}]], "Input",
 CellChangeTimes->{
  3.6685241751272106`*^9, 3.703859219292344*^9, {3.70386002150237*^9, 
   3.7038600230200443`*^9}}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a1", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "1", ",", "0", ",", 
     RowBox[{"-", "2"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"b1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "/", 
     RowBox[{"MamboNorm", "[", "v", "]"}]}], ")"}], "~", "MamboTimes", "~", 
   "v"}]}]}], "Input"],

Cell[BoxData[
 RowBox[{"b2", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "/", 
     RowBox[{"MamboNorm", "[", 
      RowBox[{"b1", "~", "MamboCross", "~", "a1"}], "]"}]}], ")"}], "~", 
   "MamboTimes", "~", 
   RowBox[{"(", 
    RowBox[{"b1", "~", "MamboCross", "~", "a1"}], ")"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"b3", "=", 
  RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Symbol", "[", 
      RowBox[{"StringJoin", "[", 
       RowBox[{"\"\<b\>\"", ",", 
        RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "]"}], "~", "MamboDot", 
     "~", 
     RowBox[{"Symbol", "[", 
      RowBox[{"StringJoin", "[", 
       RowBox[{"\"\<b\>\"", ",", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{"b1", "~", "MamboDot", "~", 
  RowBox[{"(", 
   RowBox[{"b2", "~", "MamboCross", "~", "b3"}], ")"}]}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a1", "=."}], ";", 
   RowBox[{"v", "=."}], ";", 
   RowBox[{"b1", "=."}], ";", 
   RowBox[{"b2", "=."}], ";", 
   RowBox[{"b3", "=."}], ";"}], " "}]], "Input",
 CellChangeTimes->{{3.6685243116773343`*^9, 3.6685243269837966`*^9}, 
   3.7038600332854*^9}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 3, Section 3.6", \
"Section"]], "Text",
 CellChangeTimes->{3.703859309986649*^9, 3.70385990997727*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DeclareTriads", "[", "a", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "2", ",", "1", ",", "4"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"w", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "1", ",", "1", ",", 
     RowBox[{"-", "2"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"n", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", "w", "]"}]}], ")"}], "~", "MamboTimes", "~", 
    "w"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"vpar", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"v", "~", "MamboDot", "~", "n"}], ")"}], "~", "MamboTimes", "~", 
    "n"}], "//", "MamboSimplify"}]}], "\[IndentingNewLine]", 
 RowBox[{"vperp", "=", 
  RowBox[{
   RowBox[{"n", "~", "MamboCross", "~", 
    RowBox[{"(", 
     RowBox[{"v", "~", "MamboCross", "~", "n"}], ")"}]}], "//", 
   "MamboSimplify"}]}]}], "Input",
 CellChangeTimes->{{3.703860115826085*^9, 3.703860116207782*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
  "v", "~", "MamboMinus", "~", "vpar", "~", "MamboMinus", "~", "vperp"}], "//",
   "MamboSimplify"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"v", "=."}], ";", 
   RowBox[{"w", "=."}], ";", 
   RowBox[{"n", "=."}], ";", 
   RowBox[{"vpar", "=."}], ";", 
   RowBox[{"vperp", "=."}], ";"}], " "}]], "Input",
 CellChangeTimes->{
  3.668524316308473*^9, {3.703859365217195*^9, 3.703859365360602*^9}}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "n", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"n", ",", "a1", ",", "a2", ",", "a3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"n", ",", "b1", ",", "b2", ",", "b3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"c", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"n", ",", "c1", ",", "c2", ",", "c3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "~", "MamboCross", "~", 
   RowBox[{"(", 
    RowBox[{"b", "~", "MamboCross", "~", "c"}], ")"}], "~", "MamboMinus", "~", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "~", "MamboDot", "~", "c"}], ")"}], "~", "MamboTimes", "~",
      "b"}], ")"}], "~", "MamboPlus", "~", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "~", "MamboDot", "~", "b"}], ")"}], "~", "MamboTimes", "~",
      "c"}], ")"}]}], "//", "MamboSimplify"}]}], "Input",
 CellChangeTimes->{{3.7038601446103163`*^9, 3.703860145000668*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", "=."}], ";", 
   RowBox[{"b", "=."}], ";", 
   RowBox[{"c", "=."}], ";"}], " "}]], "Input",
 CellChangeTimes->{
  3.6685243380651293`*^9, {3.703859376276849*^9, 3.703859379699772*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 4, Section 4.4", \
"Section"]], "Text",
 CellChangeTimes->{3.7038594112616463`*^9, 3.703859916057425*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B", ",", "C"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "a", ",", "1", ",", "2", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "C", ",", "a", ",", "0", ",", "2", ",", "1"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", "GlobalPointDeclarations", "\
\[IndentingNewLine]", "GlobalPointDefinitions"}], "Input"],

Cell[BoxData[
 RowBox[{"FindTranslation", "[", 
  RowBox[{"A", ",", "C"}], "]"}]], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "A", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "A", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "A", ",", "a", ",", "1", ",", "2", ",", "3"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "a", ",", "0", ",", "0", ",", "1"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "a"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", "GlobalObserverDefinitions"}], "Input"],

Cell[BoxData[
 RowBox[{"FindPosition", "[", 
  RowBox[{"W", ",", "B"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"FindCoordinates", "[", 
  RowBox[{"W", ",", "B"}], "]"}]], "Input"],

Cell[BoxData[{
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "A"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "B"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "A"}], "]"}]], "Input"],

Cell[BoxData[{
 RowBox[{"Undo", ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "B"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "B"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "A"}], "]"}]}], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", "W", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", "W", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "@@", 
  RowBox[{"Flatten", "[", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Symbol", "[", 
      RowBox[{"\"\<E\>\"", "<>", 
       RowBox[{"ToString", "[", "i", "]"}], "<>", 
       RowBox[{"ToString", "[", "j", "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "4"}], "}"}]}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "w", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{"W", ",", "W", ",", "w"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "E11", ",", "w", ",", "0", ",", 
     RowBox[{"l2", "/", "2"}], ",", 
     RowBox[{"l3", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E12", ",", "w", ",", "0", ",", 
     RowBox[{"l2", "/", "2"}], ",", 
     RowBox[{
      RowBox[{"-", "l3"}], "/", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E13", ",", "w", ",", "0", ",", 
     RowBox[{
      RowBox[{"-", "l2"}], "/", "2"}], ",", 
     RowBox[{
      RowBox[{"-", "l3"}], "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E14", ",", "w", ",", "0", ",", 
     RowBox[{
      RowBox[{"-", "l2"}], "/", "2"}], ",", 
     RowBox[{"l3", "/", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E21", ",", "w", ",", 
     RowBox[{"l1", "/", "2"}], ",", "0", ",", 
     RowBox[{"l3", "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E22", ",", "w", ",", 
     RowBox[{"l1", "/", "2"}], ",", "0", ",", 
     RowBox[{
      RowBox[{"-", "l3"}], "/", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E23", ",", "w", ",", 
     RowBox[{
      RowBox[{"-", "l1"}], "/", "2"}], ",", "0", ",", 
     RowBox[{
      RowBox[{"-", "l3"}], "/", "2"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E24", ",", "w", ",", 
     RowBox[{
      RowBox[{"-", "l1"}], "/", "2"}], ",", "0", ",", 
     RowBox[{"l3", "/", "2"}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E31", ",", "w", ",", 
     RowBox[{"l1", "/", "2"}], ",", 
     RowBox[{"l2", "/", "2"}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E32", ",", "w", ",", 
     RowBox[{"l1", "/", "2"}], ",", 
     RowBox[{
      RowBox[{"-", "l2"}], "/", "2"}], ",", "0"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E33", ",", "w", ",", 
     RowBox[{
      RowBox[{"-", "l1"}], "/", "2"}], ",", 
     RowBox[{
      RowBox[{"-", "l2"}], "/", "2"}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "E34", ",", "w", ",", 
     RowBox[{
      RowBox[{"-", "l1"}], "/", "2"}], ",", 
     RowBox[{"l2", "/", "2"}], ",", "0"}], "}"}]}], "\[IndentingNewLine]", 
  "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DefineObjects", "@@", "\[IndentingNewLine]", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"W", ",", "\"\<Block\>\"", ",", 
      RowBox[{"\"\<point\>\"", "\[Rule]", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<E1\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"\"\<xlength\>\"", "\[Rule]", "l1"}], ",", 
      RowBox[{"\"\<ylength\>\"", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}], ",", 
      RowBox[{"\"\<zlength\>\"", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}]}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "4"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DefineObjects", "@@", "\[IndentingNewLine]", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"W", ",", "\"\<Block\>\"", ",", 
      RowBox[{"\"\<point\>\"", "\[Rule]", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<E2\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"\"\<xlength\>\"", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}], ",", 
      RowBox[{"\"\<ylength\>\"", "\[Rule]", "l2"}], ",", 
      RowBox[{"\"\<zlength\>\"", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}]}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "4"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DefineObjects", "@@", "\[IndentingNewLine]", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"W", ",", "\"\<Block\>\"", ",", 
      RowBox[{"\"\<point\>\"", "\[Rule]", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<E3\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"\"\<xlength\>\"", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}], ",", 
      RowBox[{"\"\<ylength\>\"", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}], ",", 
      RowBox[{"\"\<zlength\>\"", "\[Rule]", "l3"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "4"}], "}"}]}], "]"}]}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareObservers", "[", "A", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", "A", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{"A", ",", "A", ",", "w"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"W", ",", "A", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObjects", "[", 
  RowBox[{"A", ",", "\"\<Sphere\>\"", ",", 
   RowBox[{"\"\<radius\>\"", "\[Rule]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"l1", "+", "l2", "+", "l3"}], ")"}], "/", "30"}]}], ",", 
   RowBox[{"\"\<color\>\"", "\[Rule]", "\"\<red\>\""}]}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{
   RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], ",", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<beadonblock.geo\>\""}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetTime", "[", "t", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<anims\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q1", "\[Rule]", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "l1"}], "/", "2"}], "+", 
           RowBox[{"l1", "*", "t"}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "0"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "1"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"l1", "/", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "1"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "4"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"l1", "/", "2"}], "-", 
           RowBox[{"l1", "*", 
            RowBox[{"(", 
             RowBox[{"t", "-", "4"}], ")"}]}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "4"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "5"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "l1"}], "/", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "5"}], ")"}]}]}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"q2", "\[Rule]", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "l2"}], "/", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"t", "\[GreaterEqual]", "0"}], ")"}], "*", 
            RowBox[{"(", 
             RowBox[{"t", "<", "1"}], ")"}]}], "+", 
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "4"}], ")"}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "l2"}], "/", "2"}], "+", 
           RowBox[{"l2", "*", 
            RowBox[{"(", 
             RowBox[{"t", "-", "1"}], ")"}]}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "1"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "2"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"l2", "/", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "3"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"l2", "/", "2"}], "-", 
           RowBox[{"l2", "*", 
            RowBox[{"(", 
             RowBox[{"t", "-", "3"}], ")"}]}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "3"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "4"}], ")"}]}]}]}], ",", "\[IndentingNewLine]", 
      RowBox[{"q3", "\[Rule]", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "l3"}], "/", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "<", "2"}], ")"}], "+", 
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "6"}], ")"}]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "l3"}], "/", "2"}], "+", 
           RowBox[{"l3", "*", 
            RowBox[{"(", 
             RowBox[{"t", "-", "2"}], ")"}]}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "3"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"l3", "/", "2"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "3"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "5"}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"l3", "/", "2"}], "-", 
           RowBox[{"l3", "*", 
            RowBox[{"(", 
             RowBox[{"t", "-", "5"}], ")"}]}]}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "\[GreaterEqual]", "5"}], ")"}], "*", 
         RowBox[{"(", 
          RowBox[{"t", "<", "6"}], ")"}]}]}]}]}], "}"}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"l1", "\[Rule]", "1"}], ",", 
      RowBox[{"l2", "\[Rule]", ".5"}], ",", 
      RowBox[{"l3", "\[Rule]", ".25"}]}], "}"}]}], ",", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<beadonblock.dyn\>\""}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q1", "\[Rule]", 
       RowBox[{"-", ".5"}]}], ",", 
      RowBox[{"q2", "\[Rule]", 
       RowBox[{"-", ".25"}]}], ",", 
      RowBox[{"q3", "\[Rule]", 
       RowBox[{"-", ".125"}]}]}], "}"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"l1", "\[Rule]", "1"}], ",", 
      RowBox[{"l2", "\[Rule]", ".5"}], ",", 
      RowBox[{"l3", "\[Rule]", ".25"}]}], "}"}]}], ",", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<beadonblock.dyn\>\""}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"q1", "'"}], "\[Equal]", "u1"}], ",", 
      RowBox[{
       RowBox[{"q2", "'"}], "\[Equal]", "u2"}], ",", 
      RowBox[{
       RowBox[{"q3", "'"}], "\[Equal]", "u3"}]}], "}"}]}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q1", "\[Rule]", 
       RowBox[{"-", ".5"}]}], ",", 
      RowBox[{"q2", "\[Rule]", 
       RowBox[{"-", ".25"}]}], ",", 
      RowBox[{"q3", "\[Rule]", 
       RowBox[{"-", ".125"}]}]}], "}"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"l1", "\[Rule]", "1"}], ",", 
      RowBox[{"l2", "\[Rule]", ".5"}], ",", 
      RowBox[{"l3", "\[Rule]", ".25"}]}], "}"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{"\"\<insignals\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u1", "\[Rule]", 
       RowBox[{"l1", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "0"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"t", "<", "1"}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "4"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"t", "<", "5"}], ")"}]}]}], ")"}]}]}], ",", 
      RowBox[{"u2", "\[Rule]", 
       RowBox[{"l2", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "1"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"t", "<", "2"}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "3"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"t", "<", "4"}], ")"}]}]}], ")"}]}]}], ",", 
      RowBox[{"u3", "\[Rule]", 
       RowBox[{"l3", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "2"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"t", "<", "3"}], ")"}]}], "-", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"t", "\[GreaterEqual]", "5"}], ")"}], "*", 
           RowBox[{"(", 
            RowBox[{"t", "<", "6"}], ")"}]}]}], ")"}]}]}]}], "}"}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<beadonblock.dyn\>\""}]}], 
  "]"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 4, Section 4.5", \
"Section"]], "Text",
 CellChangeTimes->{3.7038594255827293`*^9, 3.703859923721614*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "w", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"w3", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"w", ",", "3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rWA", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"w", ",", 
     RowBox[{"q1", " ", 
      RowBox[{"Sin", "[", "q2", "]"}], " ", 
      RowBox[{"Cos", "[", "q3", "]"}]}], ",", 
     RowBox[{"q1", " ", 
      RowBox[{"Sin", "[", "q2", "]"}], " ", 
      RowBox[{"Sin", "[", "q3", "]"}]}], ",", 
     RowBox[{"q1", " ", 
      RowBox[{"Cos", "[", "q2", "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"MamboNorm", "[", 
    RowBox[{"w3", "~", "MamboCross", "~", 
     RowBox[{"(", 
      RowBox[{"rWA", "~", "MamboCross", "~", "w3"}], ")"}]}], "]"}], 
   "\[Equal]", "R"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"w3", "=."}], ";", 
  RowBox[{"rWA", "=."}], ";"}]], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", "w", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"w3", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"w", ",", "3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rWA", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"w", ",", "q1", ",", "q2", ",", "q3"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{
   RowBox[{"Cos", "[", 
    FractionBox["theta", "2"], "]"}], "\[Equal]", 
   FractionBox[
    RowBox[{"rWA", "~", "MamboDot", "~", "w3"}], 
    RowBox[{
     RowBox[{"MamboNorm", "[", "rWA", "]"}], 
     RowBox[{"MamboNorm", "[", "w3", "]"}]}]]}], "]"}]}], "Input",
 CellChangeTimes->{3.703860181307602*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"w3", "=."}], ";", 
  RowBox[{"rWA", "=."}], ";"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{
    RowBox[{"q1", "^", "2"}], "+", 
    RowBox[{"q2", "^", "2"}], "+", 
    RowBox[{"q3", "^", "2"}], "-", 
    RowBox[{"R", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"q1", "-", 
       RowBox[{"3", " ", 
        RowBox[{"R", "/", "2"}]}]}], ")"}], "^", "2"}], "+", 
    RowBox[{"q2", "^", "2"}], "+", 
    RowBox[{"q3", "^", "2"}], "-", 
    RowBox[{"R", "^", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"Det", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{"f1", ",", "q1"}], "]"}], ",", 
      RowBox[{"D", "[", 
       RowBox[{"f1", ",", "q2"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{"f2", ",", "q1"}], "]"}], ",", 
      RowBox[{"D", "[", 
       RowBox[{"f2", ",", "q2"}], "]"}]}], "}"}]}], "}"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{"f1", ",", "q1"}], "]"}], ",", 
      RowBox[{"D", "[", 
       RowBox[{"f1", ",", "q3"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"D", "[", 
       RowBox[{"f2", ",", "q1"}], "]"}], ",", 
      RowBox[{"D", "[", 
       RowBox[{"f2", ",", "q3"}], "]"}]}], "}"}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f1", "=."}], ";", 
  RowBox[{"f2", "=."}], ";"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 5, Section 5.4", \
"Section"]], "Text",
 CellChangeTimes->{3.703859435614563*^9, 3.703859929329712*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeRotations", "[", 
   RowBox[{"\[Theta]", ",", "1"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeRotations", "[", 
   RowBox[{"\[Phi]", ",", "2"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeRotations", "[", 
   RowBox[{"\[Psi]", ",", "3"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeRotations", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\[Theta]", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\[Phi]", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"\[Psi]", ",", "3"}], "}"}]}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"rotmat", "=", 
    RowBox[{"MakeRotations", "[", 
     RowBox[{"\[Theta]", ",", "1"}], "]"}]}], ")"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Transpose", "[", "rotmat", "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Inverse", "[", "rotmat", "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[
 RowBox[{"Eigenvalues", "[", "rotmat", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MakeRotations", "[", 
   RowBox[{"\[Theta]", ",", "v1", ",", "v2", ",", "v3"}], "]"}], "//", 
  "MatrixForm"}]], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a1", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a2", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a3", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "2", ",", "3", ",", "1"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", "v", "]"}]}], ")"}], "~", "MamboTimes", "~", 
    "v"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", "a1"}], "]"}]}], ")"}], "~", 
    "MamboTimes", "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", "a1"}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Symbol", "[", 
      RowBox[{"\"\<a\>\"", "<>", 
       RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "~", "MamboDot", "~", 
     RowBox[{"Symbol", "[", 
      RowBox[{"\"\<b\>\"", "<>", 
       RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "//", "MatrixForm"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"a1", "=."}], ";", 
  RowBox[{"a2", "=."}], ";", 
  RowBox[{"a3", "=."}], ";", 
  RowBox[{"v", "=."}], ";", 
  RowBox[{"b1", "=."}], ";", 
  RowBox[{"b2", "=."}], ";", 
  RowBox[{"b3", "=."}], ";"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 6, Section 6.4", \
"Section"]], "Text",
 CellChangeTimes->{{3.7038594483822823`*^9, 3.703859449387772*^9}, 
   3.7038599339299603`*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "\[Theta]", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", "c", ",", 
     RowBox[{"\[Pi]", "/", "2"}], ",", "1", ",", "0", ",", "1"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", "GlobalTriadDeclarations"}], "Input"],

Cell[BoxData["GlobalTriadDefinitions"], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindRotation", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a", ",", "1", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "0", ",", "1", ",", "1"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"w", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"c", ",", "1", ",", "1", ",", "0"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"v", "~", "MamboDot", "~", "w"}]}], "Input"],

Cell[BoxData[
 RowBox[{"MamboExpress", "[", 
  RowBox[{"v", ",", "a"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"v", "~", "MamboCross", "~", "w"}]], "Input"],

Cell[BoxData[
 RowBox[{"MamboNorm", "[", "v", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"v", "=."}], ";", 
  RowBox[{"w", "=."}], ";"}]], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"A", ",", "B", ",", "W"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b", ",", "w"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B", ",", "W"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "A", ",", "a", ",", "1", ",", "2", ",", "3"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "a", ",", "3"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"w", ",", "a", ",", 
     RowBox[{"{", 
      RowBox[{"q1", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "1"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"{", 
      RowBox[{"q3", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q4", ",", "3"}], "}"}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "b"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"FindOrientation", "[", 
   RowBox[{"A", ",", "B"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[{
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "A"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "B"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "A"}], "]"}]], "Input"],

Cell[BoxData[{"Undo", "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "B"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "B"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "A"}], "]"}]}], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Symbol", "[", 
     RowBox[{"\"\<E\>\"", "<>", 
      RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Symbol", "[", 
     RowBox[{"\"\<C\>\"", "<>", 
      RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "a"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Symbol", "[", 
     RowBox[{"\"\<e\>\"", "<>", 
      RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "C1", ",", "a", ",", "l", ",", "0", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "C2", ",", "a", ",", 
     RowBox[{"l", "/", "2"}], ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", "3", "]"}], " ", 
      RowBox[{"l", "/", "2"}]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "C3", ",", "a", ",", 
     RowBox[{"l", "/", "2"}], ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", "3", "]"}], " ", 
      RowBox[{"l", "/", "6"}]}], ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", "6", "]"}], " ", 
      RowBox[{"l", "/", "3"}]}]}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "E1", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "~", "MamboTimes", "~", 
      RowBox[{"FindTranslation", "[", 
       RowBox[{"A", ",", "C1"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "E2", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "~", "MamboTimes", "~", 
      RowBox[{"FindTranslation", "[", 
       RowBox[{"A", ",", "C2"}], "]"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"A", ",", "E3", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "~", "MamboTimes", "~", 
      RowBox[{"FindTranslation", "[", 
       RowBox[{"A", ",", "C3"}], "]"}]}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "E4", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "~", "MamboTimes", "~", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "C1"}], "]"}], "~", "MamboPlus", "~", 
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "C2"}], "]"}]}], ")"}]}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"A", ",", "E5", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "~", "MamboTimes", "~", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "C1"}], "]"}], "~", "MamboPlus", "~", 
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "C3"}], "]"}]}], ")"}]}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{"A", ",", "E6", ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "/", "2"}], ")"}], "~", "MamboTimes", "~", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "C2"}], "]"}], "~", "MamboPlus", "~", 
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "C3"}], "]"}]}], ")"}]}]}], "}"}]}], 
  "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "l"}], ")"}], "~", "MamboTimes", "~", 
    RowBox[{"FindTranslation", "[", 
     RowBox[{"A", ",", "C1"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", 
        RowBox[{"MakeTranslations", "[", 
         RowBox[{"a", ",", "3"}], "]"}]}], "]"}]}], ")"}], "~", "MamboTimes", 
    "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "e1", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "i"}], "]"}], "~", "MamboDot", "~", 
      RowBox[{"Symbol", "[", 
       RowBox[{"\"\<b\>\"", "<>", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "l"}], ")"}], "~", "MamboTimes", "~", 
    RowBox[{"FindTranslation", "[", 
     RowBox[{"A", ",", "C2"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", 
        RowBox[{"MakeTranslations", "[", 
         RowBox[{"a", ",", "3"}], "]"}]}], "]"}]}], ")"}], "~", "MamboTimes", 
    "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "e2", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "i"}], "]"}], "~", "MamboDot", "~", 
      RowBox[{"Symbol", "[", 
       RowBox[{"\"\<b\>\"", "<>", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "l"}], ")"}], "~", "MamboTimes", "~", 
    RowBox[{"FindTranslation", "[", 
     RowBox[{"A", ",", "C3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", 
        RowBox[{"MakeTranslations", "[", 
         RowBox[{"a", ",", "3"}], "]"}]}], "]"}]}], ")"}], "~", "MamboTimes", 
    "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "e3", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "i"}], "]"}], "~", "MamboDot", "~", 
      RowBox[{"Symbol", "[", 
       RowBox[{"\"\<b\>\"", "<>", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "l"}], ")"}], "~", "MamboTimes", "~", 
    RowBox[{"FindTranslation", "[", 
     RowBox[{"C1", ",", "C2"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", 
        RowBox[{"MakeTranslations", "[", 
         RowBox[{"a", ",", "3"}], "]"}]}], "]"}]}], ")"}], "~", "MamboTimes", 
    "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "e4", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "i"}], "]"}], "~", "MamboDot", "~", 
      RowBox[{"Symbol", "[", 
       RowBox[{"\"\<b\>\"", "<>", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "l"}], ")"}], "~", "MamboTimes", "~", 
    RowBox[{"FindTranslation", "[", 
     RowBox[{"C1", ",", "C3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", 
        RowBox[{"MakeTranslations", "[", 
         RowBox[{"a", ",", "3"}], "]"}]}], "]"}]}], ")"}], "~", "MamboTimes", 
    "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "e5", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "i"}], "]"}], "~", "MamboDot", "~", 
      RowBox[{"Symbol", "[", 
       RowBox[{"\"\<b\>\"", "<>", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", "l"}], ")"}], "~", "MamboTimes", "~", 
    RowBox[{"FindTranslation", "[", 
     RowBox[{"C2", ",", "C3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"b1", "~", "MamboCross", "~", 
        RowBox[{"MakeTranslations", "[", 
         RowBox[{"a", ",", "3"}], "]"}]}], "]"}]}], ")"}], "~", "MamboTimes", 
    "~", 
    RowBox[{"(", 
     RowBox[{"b1", "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "e6", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "i"}], "]"}], "~", "MamboDot", "~", 
      RowBox[{"Symbol", "[", 
       RowBox[{"\"\<b\>\"", "<>", 
        RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"j", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DefineObjects", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"A", ",", "\"\<Block\>\"", ",", 
      RowBox[{"\"\<point\>\"", "\[Rule]", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<E\>\"", "<>", 
         RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
      RowBox[{"\"\<orient\>\"", "\[Rule]", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<e\>\"", "<>", 
         RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
      RowBox[{"\"\<xlength\>\"", "\[Rule]", "l"}], ",", 
      RowBox[{"\"\<ylength\>\"", "\[Rule]", 
       RowBox[{"l", "/", "10"}]}], ",", 
      RowBox[{"\"\<zlength\>\"", "\[Rule]", 
       RowBox[{"l", "/", "10"}]}], ",", 
      RowBox[{"\"\<color\>\"", "->", "\"\<green\>\""}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DefineObjects", "[", 
  RowBox[{"W", ",", "\"\<Block\>\"", ",", 
   RowBox[{"\"\<xlength\>\"", "\[Rule]", 
    RowBox[{"5", " ", "l"}]}], ",", 
   RowBox[{"\"\<ylength\>\"", "\[Rule]", 
    RowBox[{"5", " ", "l"}]}], ",", 
   RowBox[{"\"\<zlength\>\"", "\[Rule]", 
    RowBox[{"l", "/", "100"}]}], ",", 
   RowBox[{"\"\<color\>\"", "->", "\"\<white\>\""}]}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DefinePoints", "[", 
  RowBox[{"W", ",", "A", ",", "NullVector"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"w", ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"q1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q2", ",", 
     RowBox[{"1", "/", "2"}], ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", "3", "]"}], "/", "2"}], ",", "0"}], "}"}]}], 
  "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"GeometryOutput", "[", 
  RowBox[{
   RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], ",", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<flippingtetrahedron.geo\>\""}]}], 
  "]"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetTime", "[", "t", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<anims\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q1", "\[Rule]", 
       RowBox[{
        RowBox[{"\[Pi]", "/", "3"}], " ", 
        RowBox[{"Floor", "[", "t", "]"}]}]}], ",", 
      RowBox[{"q2", "\[Rule]", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"ArcCos", "[", 
           RowBox[{"1", "/", "3"}], "]"}], "-", "\[Pi]"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"t", "-", 
          RowBox[{"Floor", "[", "t", "]"}]}], ")"}]}]}]}], "}"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"l", "\[Rule]", "1"}], "}"}]}], ",", "\[IndentingNewLine]", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<flippingtetrahedron.dyn\>\""}]}], 
  "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"b1", "=."}], ";", 
  RowBox[{"b2", "=."}], ";", 
  RowBox[{"b3", "=."}], ";"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 6, Section 6.5", \
"Section"]], "Text",
 CellChangeTimes->{3.7038594719599447`*^9, 3.703859941241591*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "A", ",", "P"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "a"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "A", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "P", ",", "a", ",", 
     RowBox[{"rho", " ", 
      RowBox[{"Cos", "[", "q7", "]"}]}], ",", 
     RowBox[{"rho", " ", 
      RowBox[{"Sin", "[", "q7", "]"}]}], ",", "0"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"w", ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"q4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q6", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"W", ",", "P"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"A", ",", "P"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"w", ",", "3"}], "]"}], "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "3"}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Factor", "[", 
   RowBox[{"Det", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"D", "[", 
         RowBox[{"f1", ",", "q3"}], "]"}], ",", 
        RowBox[{"D", "[", 
         RowBox[{"f1", ",", "q6"}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"D", "[", 
         RowBox[{"f2", ",", "q3"}], "]"}], ",", 
        RowBox[{"D", "[", 
         RowBox[{"f2", ",", "q6"}], "]"}]}], "}"}]}], "}"}], "]"}], "]"}], "//",
   "Simplify"}]}], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "A", ",", "P", ",", "Pp"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "a"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "A", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "P", ",", "a", ",", 
     RowBox[{"rho", " ", 
      RowBox[{"Cos", "[", "q7", "]"}]}], ",", 
     RowBox[{"rho", " ", 
      RowBox[{"Sin", "[", "q7", "]"}]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"W", ",", "Pp", ",", "w", ",", 
     RowBox[{"R", " ", 
      RowBox[{"Sin", "[", "q8", "]"}], " ", 
      RowBox[{"Cos", "[", "q9", "]"}]}], ",", 
     RowBox[{"R", " ", 
      RowBox[{"Sin", "[", "q8", "]"}], " ", 
      RowBox[{"Sin", "[", "q9", "]"}]}], ",", 
     RowBox[{"R", " ", 
      RowBox[{"Cos", "[", "q8", "]"}]}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"w", ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"q4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q6", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "P"}], "]"}], "~", "MamboMinus", "~", 
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "Pp"}], "]"}]}], ")"}], "~", "MamboDot", "~", 
     RowBox[{"MakeTranslations", "[", 
      RowBox[{"w", ",", "1"}], "]"}]}], "\[Equal]", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "P"}], "]"}], "~", "MamboMinus", "~", 
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "Pp"}], "]"}]}], ")"}], "~", "MamboDot", "~", 
     RowBox[{"MakeTranslations", "[", 
      RowBox[{"w", ",", "2"}], "]"}]}], "\[Equal]", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f3", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "P"}], "]"}], "~", "MamboMinus", "~", 
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "Pp"}], "]"}]}], ")"}], "~", "MamboDot", "~", 
     RowBox[{"MakeTranslations", "[", 
      RowBox[{"w", ",", "3"}], "]"}]}], "\[Equal]", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f4", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"FindTranslation", "[", 
        RowBox[{"W", ",", "Pp"}], "]"}], "~", "MamboCross", "~", 
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"a", ",", "3"}], "]"}]}], ")"}], "~", "MamboDot", "~", 
     RowBox[{"FindTranslation", "[", 
      RowBox[{"A", ",", "P"}], "]"}]}], "\[Equal]", "0"}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f1", "=."}], ";", 
  RowBox[{"f2", "=."}], ";", 
  RowBox[{"f3", "=."}], ";", 
  RowBox[{"f4", "=."}], ";"}]], "Input",
 CellChangeTimes->{{3.7038602140422077`*^9, 3.7038602221062803`*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 7, Section 7.3", \
"Section"]], "Text",
 CellChangeTimes->{3.703859479918378*^9, 3.7038599468424883`*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "Arear", ",", "Afront", ",", "Aframe", ",", "Asteer"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "Arear", ",", "Afront", ",", "Aframe", ",", "Asteer"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "arear", ",", "afront", ",", "aframe", ",", "asteer"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Arear", ",", "Arear", ",", "arear"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Afront", ",", "Afront", ",", "afront"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Aframe", ",", "Aframe", ",", "aframe"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Asteer", ",", "Asteer", ",", "asteer"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"W", ",", "Aframe", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], 
   "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"w", ",", "aframe", ",", 
    RowBox[{"{", 
     RowBox[{"q4", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q5", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q6", ",", "3"}], "}"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"Aframe", ",", "Arear", ",", "NullVector"}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"aframe", ",", "arear", ",", 
    RowBox[{"{", 
     RowBox[{"q7", ",", "3"}], "}"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "p1", ",", "p2", ",", "0"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aframe1", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aframe2", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aframe3", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", "v", "]"}]}], ")"}], "~", "MamboTimes", "~", 
    "v"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"aframe3", "~", "MamboCross", "~", " ", "b1"}], "]"}]}], ")"}],
     "~", "MamboTimes", "~", 
    RowBox[{"(", 
     RowBox[{"aframe3", "~", "MamboCross", "~", " ", "b1"}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", " ", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"aframe", ",", "asteer", ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<aframe\>\"", "<>", 
         RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "~", "MamboDot", "~", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<b\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "3"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"q8", ",", "1"}], "}"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"Aframe", ",", "Asteer", ",", 
    RowBox[{"{", 
     RowBox[{"aframe", ",", "p3", ",", "p4", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"asteer", ",", "p5", ",", "p6", ",", "0"}], "}"}]}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"Asteer", ",", "Afront", ",", "NullVector"}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"asteer", ",", "afront", ",", "q9", ",", "3"}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Aframe"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Arear", ",", "Aframe"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Aframe", ",", "Asteer"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Asteer", ",", "Afront"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"Prear", ",", "Pfront"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Arear", ",", "Prear", ",", "arear", ",", 
     RowBox[{"R", "*", 
      RowBox[{"Cos", "[", "q10", "]"}]}], ",", 
     RowBox[{"R", "*", 
      RowBox[{"Sin", "[", "q10", "]"}]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Afront", ",", "Pfront", ",", "afront", ",", 
     RowBox[{"R", "*", 
      RowBox[{"Cos", "[", "q11", "]"}]}], ",", 
     RowBox[{"R", "*", 
      RowBox[{"Sin", "[", "q11", "]"}]}], ",", "0"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"FindTranslation", "[", 
      RowBox[{"Arear", ",", "Prear"}], "]"}], "~", "MamboDot", "~", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"w", ",", "3"}], "]"}], "~", "MamboCross", "~", " ", 
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"arear", ",", "3"}], "]"}]}], ")"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"W", ",", "Prear"}], "]"}], "~", "MamboDot", "~", " ", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";", 
  RowBox[{"f3", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"FindTranslation", "[", 
      RowBox[{"Afront", ",", "Pfront"}], "]"}], "~", "MamboDot", "~", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"w", ",", "3"}], "]"}], "~", "MamboCross", "~", " ", 
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"afront", ",", "3"}], "]"}]}], ")"}]}], "]"}]}], ";", 
  RowBox[{"f4", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"W", ",", "Pfront"}], "]"}], "~", "MamboDot", "~", " ", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"f1", ",", "f2", ",", "f3", ",", "f4"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"q1", "\[Rule]", "0"}], ",", 
    RowBox[{"q2", "\[Rule]", "0"}], ",", 
    RowBox[{"q3", "\[Rule]", ".4"}], ",", 
    RowBox[{"q4", "\[Rule]", "0"}], ",", 
    RowBox[{"q5", "\[Rule]", "1"}], ",", 
    RowBox[{"q6", "\[Rule]", "0"}], ",", 
    RowBox[{"q7", "\[Rule]", "0"}], ",", 
    RowBox[{"q8", "\[Rule]", ".7"}], ",", 
    RowBox[{"q9", "\[Rule]", "0"}], ",", 
    RowBox[{"q10", "\[Rule]", 
     RowBox[{"-", "1.5"}]}], ",", 
    RowBox[{"q11", "\[Rule]", "3.14"}], ",", 
    RowBox[{"p1", "\[Rule]", 
     RowBox[{"-", ".348968837"}]}], ",", 
    RowBox[{"p2", "\[Rule]", ".937134317"}], ",", 
    RowBox[{"p3", "\[Rule]", "1.296859217"}], ",", 
    RowBox[{"p4", "\[Rule]", ".6774200135"}], ",", 
    RowBox[{"p5", "\[Rule]", 
     RowBox[{"-", ".7601008180"}]}], ",", 
    RowBox[{"p6", "\[Rule]", 
     RowBox[{"-", ".10"}]}], ",", 
    RowBox[{"R", "\[Rule]", ".5"}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"FindRoot", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<f\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"q1", "\[Rule]", "0"}], ",", 
          RowBox[{"q2", "\[Rule]", "0"}], ",", 
          RowBox[{"q4", "\[Rule]", "0"}], ",", 
          RowBox[{"q5", "\[Rule]", "1"}], ",", 
          RowBox[{"q7", "\[Rule]", "0"}], ",", 
          RowBox[{"q8", "\[Rule]", ".7"}], ",", 
          RowBox[{"q9", "\[Rule]", "0"}], ",", 
          RowBox[{"p1", "\[Rule]", 
           RowBox[{"-", ".348968837"}]}], ",", 
          RowBox[{"p2", "\[Rule]", ".937134317"}], ",", 
          RowBox[{"p3", "\[Rule]", "1.296859217"}], ",", 
          RowBox[{"p4", "\[Rule]", ".6774200135"}], ",", 
          RowBox[{"p5", "\[Rule]", 
           RowBox[{"-", ".7601008180"}]}], ",", 
          RowBox[{"p6", "\[Rule]", 
           RowBox[{"-", ".10"}]}], ",", 
          RowBox[{"R", "\[Rule]", ".5"}]}], "}"}]}], ")"}], "\[Equal]", "0"}],
      ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "4"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"q3", ",", ".4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q6", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q10", ",", 
     RowBox[{"-", "1.5"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q11", ",", "3.14"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"v", "=."}], ";", 
  RowBox[{"aframe1", "=."}], ";", 
  RowBox[{"aframe2", "=."}], ";", 
  RowBox[{"aframe3", "=."}], ";", 
  RowBox[{"b1", "=."}], ";", 
  RowBox[{"b2", "=."}], ";", 
  RowBox[{"b3", "=."}], ";", 
  RowBox[{"f1", "=."}], ";", 
  RowBox[{"f2", "=."}], ";", 
  RowBox[{"f3", "=."}], ";", 
  RowBox[{"f4", "=."}], ";"}]], "Input",
 CellChangeTimes->{{3.7038602342514353`*^9, 3.703860256899501*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 7, Section 7.4", \
"Section"]], "Text",
 CellChangeTimes->{3.703859569315456*^9, 3.7038599522100267`*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{
  "W", ",", "Base", ",", "Lamp", ",", "Bracket", ",", "UpperBeam", ",", 
   "LowerBeam", ",", "MiddleBeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{
  "W", ",", "Base", ",", "Lamp", ",", "Bracket", ",", "UpperBeam", ",", 
   "LowerBeam", ",", "MiddleBeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{
  "w", ",", "base", ",", "lamp", ",", "bracket", ",", "upperbeam", ",", 
   "lowerbeam", ",", "middlebeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Base"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "LowerBeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"LowerBeam", ",", "Bracket"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "Lamp"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "MiddleBeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "UpperBeam"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "Base", ",", "base"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"LowerBeam", ",", "LowerBeam", ",", "lowerbeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"MiddleBeam", ",", "MiddleBeam", ",", "middlebeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"UpperBeam", ",", "UpperBeam", ",", "upperbeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "Bracket", ",", "bracket"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Lamp", ",", "Lamp", ",", "lamp"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Base", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Base", ",", "LowerBeam", ",", "base", ",", "0", ",", "0", ",", "p1"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Base", ",", "MiddleBeam", ",", "base", ",", "0", ",", "0", ",", "p2"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Base", ",", "UpperBeam", ",", "base", ",", "0", ",", "0", ",", "p3"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "LowerBeam", ",", "Bracket", ",", "lowerbeam", ",", "0", ",", "0", ",", 
     "p4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Bracket", ",", "Lamp", ",", "bracket", ",", "0", ",", "0", ",", "p5"}], 
    "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"w", ",", "base", ",", 
     RowBox[{"{", 
      RowBox[{"q4", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q5", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q6", ",", "3"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"base", ",", "lowerbeam", ",", "q7", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"lowerbeam", ",", "bracket", ",", "q10", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"bracket", ",", "lamp", ",", "q11", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"base", ",", "middlebeam", ",", "q8", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"base", ",", "upperbeam", ",", "q9", ",", "2"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"GeometryOutput", "[", 
  RowBox[{"\"\<main\>\"", "\[Rule]", "W"}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"HUpperBeam", ",", "HBracket"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "UpperBeam", ",", "HUpperBeam", ",", "upperbeam", ",", "0", ",", "0", ",",
      "p6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "HBracket", ",", "bracket", ",", 
     RowBox[{"-", "p7"}], ",", "0", ",", "p8"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"HBracket", ",", "HUpperBeam"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"base", ",", "1"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"HBracket", ",", "HUpperBeam"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"base", ",", "3"}], "]"}]}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"f1", ",", "f2"}], "}"}], "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"p1", "\[Rule]", "11"}], ",", 
    RowBox[{"p2", "\[Rule]", "12.5"}], ",", 
    RowBox[{"p3", "\[Rule]", "14"}], ",", 
    RowBox[{"p4", "\[Rule]", "45"}], ",", 
    RowBox[{"p5", "\[Rule]", "3.5"}], ",", 
    RowBox[{"p6", "\[Rule]", "45"}], ",", 
    RowBox[{"p8", "\[Rule]", "2.6"}], ",", 
    RowBox[{"p7", "\[Rule]", "1.5"}], ",", 
    RowBox[{"q1", "\[Rule]", "0"}], ",", 
    RowBox[{"q2", "\[Rule]", "0"}], ",", 
    RowBox[{"q3", "\[Rule]", "0"}], ",", 
    RowBox[{"q4", "\[Rule]", "0"}], ",", 
    RowBox[{"q5", "\[Rule]", "0"}], ",", 
    RowBox[{"q6", "\[Rule]", "0"}], ",", 
    RowBox[{"q7", "\[Rule]", "1.2"}], ",", 
    RowBox[{"q8", "\[Rule]", "1.8"}], ",", 
    RowBox[{"q9", "\[Rule]", "1.2"}], ",", 
    RowBox[{"q10", "\[Rule]", 
     RowBox[{"-", ".67"}]}], ",", 
    RowBox[{"q11", "\[Rule]", "2"}]}], "}"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"FindRoot", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<f\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"p1", "\[Rule]", "11"}], ",", 
          RowBox[{"p2", "\[Rule]", "12.5"}], ",", 
          RowBox[{"p3", "\[Rule]", "14"}], ",", 
          RowBox[{"p4", "\[Rule]", "45"}], ",", 
          RowBox[{"p5", "\[Rule]", "3.5"}], ",", 
          RowBox[{"p6", "\[Rule]", "45"}], ",", 
          RowBox[{"p8", "\[Rule]", "2.6"}], ",", 
          RowBox[{"p7", "\[Rule]", "1.5"}], ",", 
          RowBox[{"q1", "\[Rule]", "0"}], ",", 
          RowBox[{"q2", "\[Rule]", "0"}], ",", 
          RowBox[{"q3", "\[Rule]", "0"}], ",", 
          RowBox[{"q4", "\[Rule]", "0"}], ",", 
          RowBox[{"q5", "\[Rule]", "0"}], ",", 
          RowBox[{"q6", "\[Rule]", "0"}], ",", 
          RowBox[{"q8", "\[Rule]", "1.8"}], ",", 
          RowBox[{"q9", "\[Rule]", "1.2"}], ",", 
          RowBox[{"q11", "\[Rule]", "2"}]}], "}"}]}], ")"}], "\[Equal]", 
      "0"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"q7", ",", "1.2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q10", ",", 
     RowBox[{"-", ".67"}]}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"f1", "=."}], ";", 
  RowBox[{"f2", "=."}], ";"}]], "Input",
 CellChangeTimes->{{3.7038602632000313`*^9, 3.703860265923492*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 8, Section 8.5", \
"Section"]], "Text",
 CellChangeTimes->{3.703859578491559*^9, 3.703859956690628*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2", ",", "q3"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DiffTime", "[", 
  RowBox[{"q1", "-", 
   RowBox[{"q2", " ", 
    RowBox[{"Sin", "[", "q3", "]"}]}]}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"DiffTime", "[", 
  RowBox[{"{", 
   RowBox[{"q1", ",", 
    RowBox[{"q2", "-", 
     RowBox[{"q3", " ", 
      RowBox[{"Log", "[", "q1", "]"}]}]}]}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiffTime", "[", 
   RowBox[{"MakeRotations", "[", 
    RowBox[{"q1", ",", "3"}], "]"}], "]"}], "//", "MatrixForm"}]], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareTriads", "[", "a", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DiffTime", "[", 
  RowBox[{
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"a", ",", "q1", ",", "q2", ",", "q3"}], "]"}], ",", "a"}], 
  "]"}]}], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "q1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"b", ",", "c", ",", "q2", ",", 
     RowBox[{"Cos", "[", "t", "]"}], ",", 
     RowBox[{"Sin", "[", "t", "]"}], ",", "0"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetTime", "[", "t", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2"}], "]"}]}], "Input",
 CellChangeTimes->{{3.668364916465845*^9, 3.6683649247993603`*^9}}],

Cell[BoxData[
 RowBox[{"AngularVelocity", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Simplify", "[", 
  RowBox[{"AngularVelocity", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "]"}]], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareObservers", "[", "A", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B", ",", "C"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{"A", ",", "A", ",", "a"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "a", ",", "q3", ",", "q4", ",", "q5"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "C", ",", "b", ",", "0", ",", "q6", ",", "0"}], "}"}]}],
   "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q3", ",", "q4", ",", "q5", ",", "q6"}], "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"LinearVelocity", "[", 
  RowBox[{"A", ",", "B"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"LinearVelocity", "[", 
  RowBox[{"A", ",", "C"}], "]"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 8, Section 8.6", \
"Section"]], "Text",
 CellChangeTimes->{3.7038595867613697`*^9, 3.703859963778841*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"A", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B", ",", "P"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "b"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "a", ",", "r1", ",", "r2", ",", "r3"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "P", ",", "b", ",", "l", ",", "0", ",", "0"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "b", ",", "q1", ",", "v1", ",", "v2", ",", "v3"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", "q1", "]"}], "\[IndentingNewLine]", 
 RowBox[{"LinearVelocity", "[", 
  RowBox[{"A", ",", "P"}], "]"}]}], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"A", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B", ",", "P"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "b"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "B", ",", "a", ",", "r1", ",", "r2", ",", "r3"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "P", ",", "b", ",", 
     RowBox[{"s", "/", "2"}], ",", 
     RowBox[{"s", "/", "2"}], ",", "0"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"theta", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"phi", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"psi", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"theta", ",", "phi", ",", "psi"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"LinearVelocity", "[", 
  RowBox[{"A", ",", "P"}], "]"}]}], "Input"],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"A", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B", ",", "P"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "b"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"B", ",", "A", ",", "b", ",", "0", ",", "0", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "P", ",", "a", ",", 
     RowBox[{"q1", " ", 
      RowBox[{"Sin", "[", "q2", "]"}], " ", 
      RowBox[{"Cos", "[", "q3", "]"}]}], ",", 
     RowBox[{"q1", " ", 
      RowBox[{"Sin", "[", "q2", "]"}], " ", 
      RowBox[{"Sin", "[", "q3", "]"}]}], ",", 
     RowBox[{"q1", " ", 
      RowBox[{"Cos", "[", "q2", "]"}]}]}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"b", ",", "a", ",", 
   RowBox[{"omega", " ", "t"}], ",", "3"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"SetTime", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2", ",", "q3"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"LinearVelocity", "[", 
  RowBox[{"B", ",", "P"}], "]"}]}], "Input",
 CellChangeTimes->{{3.668364949650244*^9, 3.6683649669341464`*^9}}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", "A", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"A", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"a", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{"A", ",", "A", ",", "a"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"A", ",", "B", ",", "a", ",", "q1", ",", "q2", ",", "R"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"q3", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q5", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", "q5"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"vel", "=", 
   RowBox[{
    RowBox[{"LinearVelocity", "[", 
     RowBox[{"A", ",", "B"}], "]"}], "~", "MamboPlus", "~", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"AngularVelocity", "[", 
       RowBox[{"a", ",", "b"}], "]"}], "~", "MamboCross", "~", 
      RowBox[{"MakeTranslations", "[", 
       RowBox[{"a", ",", "0", ",", "0", ",", 
        RowBox[{"-", "R"}]}], "]"}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MamboExpress", "[", 
   RowBox[{"vel", ",", "a"}], "]"}], "//", "MamboSimplify"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vel", "=."}], ";"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 9, Section 9.3", \
"Section"]], "Text",
 CellChangeTimes->{3.7038595953387413`*^9, 3.703859969155044*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"q1", "'"}], " ", "q2"}], "+", 
       RowBox[{"q3", "'"}]}], "\[Equal]", "0"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"q2", "'"}], " ", 
       SuperscriptBox["q1", "2"]}], "\[Equal]", 
      RowBox[{"Cos", "[", "t", "]"}]}], ",", 
     RowBox[{
      RowBox[{"q1", "'"}], "\[Equal]", "u1"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetTime", "[", "t", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", "kde"}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"q1", ",", "q2", ",", "q3"}], "}"}]}], ",", 
   RowBox[{"\"\<insignals\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"u1", "\[Rule]", "1"}], "}"}]}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.6683649828675814`*^9, 3.668364984944786*^9}}],

Cell[BoxData[
 RowBox[{"Det", "[", 
  RowBox[{
   RowBox[{"CoefficientArrays", "[", 
    RowBox[{"kde", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q1", "'"}], ",", 
       RowBox[{"q2", "'"}], ",", 
       RowBox[{"q3", "'"}]}], "}"}]}], "]"}], "[", 
   RowBox[{"[", "2", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", "kde"}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"q1", ",", "q2", ",", "q3"}], "}"}]}], ",", "\"\<checksings\>\"",
    ",", 
   RowBox[{"\"\<insignals\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"u1", "\[Rule]", "1"}], "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"kde", "=."}], ";"}]], "Input",
 CellChangeTimes->{{3.703860281775694*^9, 3.703860284236039*^9}}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "B", ",", "P1", ",", "P2"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "B", ",", "b"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "B", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "P1", ",", "b", ",", "p11", ",", "p12", ",", "p13"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"B", ",", "P2", ",", "b", ",", "p21", ",", "p22", ",", "p23"}], 
    "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"w", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"q4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q6", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", "q5", ",", "q6"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"k", "~", "MamboTimes", "~", 
             RowBox[{"LinearVelocity", "[", 
              RowBox[{"W", ",", "P1"}], "]"}]}], ")"}], "~", "MamboPlus", "~", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "-", "k"}], ")"}], "~", "MamboTimes", "~", 
             RowBox[{"LinearVelocity", "[", 
              RowBox[{"W", ",", "P2"}], "]"}]}], ")"}]}], ")"}], "~", 
         "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], " ", ",", 
     "\[IndentingNewLine]", "\t", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"w", ",", "b"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"b", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Simplify", "[", 
  RowBox[{"Det", "[", 
   RowBox[{
    RowBox[{"CoefficientArrays", "[", 
     RowBox[{"kde", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q1", "'"}], ",", 
        RowBox[{"q2", "'"}], ",", 
        RowBox[{"q3", "'"}], ",", 
        RowBox[{"q4", "'"}], ",", 
        RowBox[{"q5", "'"}], ",", 
        RowBox[{"q6", "'"}]}], "}"}]}], "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}], "]"}]}], "Input",
 CellChangeTimes->{{3.6683650075749655`*^9, 3.668365008215995*^9}}],

Cell[BoxData[{
 RowBox[{"SetTime", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", "kde"}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", "q5", ",", "q6"}], 
     "}"}]}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.668365011490178*^9, 3.6683650121802063`*^9}}],

Cell[BoxData[
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", "kde"}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", "q5", ",", "q6"}], 
     "}"}]}], ",", 
   RowBox[{"\"\<insignals\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u1", "\[Rule]", 
       RowBox[{"Cos", "[", "t", "]"}]}], ",", 
      RowBox[{"u2", "\[Rule]", 
       RowBox[{"-", "1"}]}], ",", 
      RowBox[{"u3", "\[Rule]", "0"}], ",", 
      RowBox[{"k", "\[Rule]", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Sin", "[", 
          RowBox[{"2", " ", "\[Pi]", " ", 
           RowBox[{"t", "/", "T"}]}], "]"}], "\[GreaterEqual]", "0"}], 
        ")"}]}]}], "}"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
     "p11", ",", "p12", ",", "p13", ",", "p21", ",", "p22", ",", "p23", ",", 
      "T"}], "}"}]}], ",", 
   RowBox[{"\"\<filename\>\"", "->", "\"\<switching.dyn\>\""}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"kde", "=."}], ";"}]], "Input",
 CellChangeTimes->{{3.70386028759761*^9, 3.70386029126051*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 9, Section 9.4", \
"Section"]], "Text",
 CellChangeTimes->{3.703859602556952*^9, 3.703859974643141*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "A"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "a"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"A", ",", "A", ",", "a"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"W", ",", "A", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"w", ",", "a", ",", 
   RowBox[{"{", 
    RowBox[{"q4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q6", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", "q5", ",", "q6"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"FindTranslation", "[", 
    RowBox[{"W", ",", "A"}], "]"}], "~", "MamboDot", "~", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"w", ",", "3"}], "]"}]}], "\[LessEqual]", "rho"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclarePoints", "[", "P", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"A", ",", "P", ",", "a", ",", 
   RowBox[{"rho", " ", 
    RowBox[{"Sin", "[", "q7", "]"}], " ", 
    RowBox[{"Cos", "[", "q8", "]"}]}], ",", 
   RowBox[{"rho", " ", 
    RowBox[{"Sin", "[", "q7", "]"}], 
    RowBox[{"Sin", "[", "q8", "]"}]}], ",", 
   RowBox[{"rho", " ", 
    RowBox[{"Cos", "[", "q7", "]"}]}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q7", ",", "q8"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"confconst1", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"W", ",", "P"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"motionconst1", "=", 
   RowBox[{"DiffTime", "[", "confconst1", "]"}]}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"LinearVelocity", "[", 
          RowBox[{"W", ",", "A"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"w", ",", "a"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"a", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"i", "+", "3"}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"LinearVelocity", "[", 
          RowBox[{"W", ",", "P"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "1"}], "]"}]}], "\[Equal]", "u7"}], ",", 
       RowBox[{"motionconst1", "\[Equal]", "0"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{
    RowBox[{"CoefficientArrays", "[", 
     RowBox[{"kde", ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<q\>\"", "<>", 
           RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "'"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "8"}], "}"}]}], "]"}]}], "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}], "//", "Simplify"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"confconst2", "=", 
   RowBox[{
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"a", ",", 
      RowBox[{"rho", " ", 
       RowBox[{"Cos", "[", "q7", "]"}], " ", 
       RowBox[{"Cos", "[", "q8", "]"}]}], ",", 
      RowBox[{"rho", " ", 
       RowBox[{"Cos", "[", "q7", "]"}], " ", 
       RowBox[{"Sin", "[", "q8", "]"}]}], ",", 
      RowBox[{
       RowBox[{"-", "rho"}], " ", 
       RowBox[{"Sin", "[", "q7", "]"}]}]}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"confconst3", "=", 
   RowBox[{
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"a", ",", 
      RowBox[{
       RowBox[{"-", "rho"}], " ", 
       RowBox[{"Sin", "[", "q7", "]"}], 
       RowBox[{"Sin", "[", "q8", "]"}]}], ",", 
      RowBox[{"rho", " ", 
       RowBox[{"Sin", "[", "q7", "]"}], 
       RowBox[{"Cos", "[", "q8", "]"}]}], ",", "0"}], "]"}], "~", "MamboDot", 
    "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"motionconst2", "=", 
   RowBox[{"DiffTime", "[", "confconst2", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"motionconst3", "=", 
   RowBox[{"DiffTime", "[", "confconst3", "]"}]}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"LinearVelocity", "[", 
          RowBox[{"W", ",", "A"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "2"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"w", ",", "a"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"a", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"i", "+", "3"}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"motionconst1", "\[Equal]", "0"}], ",", 
       RowBox[{"motionconst2", "\[Equal]", "0"}], ",", 
       RowBox[{"motionconst3", "\[Equal]", "0"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{
    RowBox[{"CoefficientArrays", "[", 
     RowBox[{"kde", ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<q\>\"", "<>", 
           RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "'"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "8"}], "}"}]}], "]"}]}], "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}], "//", "Simplify"}]}], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"motionconst4", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"LinearVelocity", "[", 
       RowBox[{"W", ",", "A"}], "]"}], "~", "MamboPlus", "~", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"AngularVelocity", "[", 
         RowBox[{"w", ",", "a"}], "]"}], "~", "MamboCross", "~", 
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "P"}], "]"}]}], ")"}]}], ")"}], "~", "MamboDot", 
    "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "1"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"motionconst5", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"LinearVelocity", "[", 
       RowBox[{"W", ",", "A"}], "]"}], "~", "MamboPlus", "~", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"AngularVelocity", "[", 
         RowBox[{"w", ",", "a"}], "]"}], "~", "MamboCross", "~", 
        RowBox[{"FindTranslation", "[", 
         RowBox[{"A", ",", "P"}], "]"}]}], ")"}]}], ")"}], "~", "MamboDot", 
    "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "2"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"w", ",", "a"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"a", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"i", "+", "3"}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"motionconst1", "\[Equal]", "0"}], ",", 
       RowBox[{"motionconst2", "\[Equal]", "0"}], ",", 
       RowBox[{"motionconst3", "\[Equal]", "0"}], ",", 
       RowBox[{"motionconst4", "\[Equal]", "0"}], ",", 
       RowBox[{"motionconst5", "\[Equal]", "0"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Det", "[", 
   RowBox[{
    RowBox[{"CoefficientArrays", "[", 
     RowBox[{"kde", ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<q\>\"", "<>", 
           RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "'"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "8"}], "}"}]}], "]"}]}], "]"}], "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}], "//", "Simplify"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"confconst1", "=."}], ";", 
  RowBox[{"confconst2", "=."}], ";", 
  RowBox[{"confconst3", "=."}], ";", 
  RowBox[{"motionconst1", "=."}], ";", 
  RowBox[{"motionconst2", "=."}], ";", 
  RowBox[{"motionconst3", "=."}], ";", 
  RowBox[{"motionconst4", "=."}], ";", 
  RowBox[{"motionconst5", "=."}], ";", 
  RowBox[{"kde", "=."}], ";"}]], "Input",
 CellChangeTimes->{{3.703860307222981*^9, 3.7038603099889174`*^9}}],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 10, Section \
10.3", "Section"]], "Text",
 CellChangeTimes->{
  3.703859609917042*^9, {3.703859979459427*^9, 3.703859980426951*^9}}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{"W", ",", "Arear", ",", "Afront", ",", "Aframe", ",", "Asteer"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"W", ",", "Arear", ",", "Afront", ",", "Aframe", ",", "Asteer"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"w", ",", "arear", ",", "afront", ",", "aframe", ",", "asteer"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Arear", ",", "Arear", ",", "arear"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Afront", ",", "Afront", ",", "afront"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Aframe", ",", "Aframe", ",", "aframe"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Asteer", ",", "Asteer", ",", "asteer"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"W", ",", "Aframe", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], 
   "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"w", ",", "aframe", ",", 
    RowBox[{"{", 
     RowBox[{"q4", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q5", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q6", ",", "3"}], "}"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"Aframe", ",", "Arear", ",", "NullVector"}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"aframe", ",", "arear", ",", 
    RowBox[{"{", 
     RowBox[{"q7", ",", "3"}], "}"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"v", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "p1", ",", "p2", ",", "0"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aframe1", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aframe2", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "2"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"aframe3", "=", 
   RowBox[{"MakeTranslations", "[", 
    RowBox[{"aframe", ",", "3"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b1", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", "v", "]"}]}], ")"}], "~", "MamboTimes", "~", 
    "v"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b2", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "/", 
      RowBox[{"MamboNorm", "[", 
       RowBox[{"aframe3", "~", "MamboCross", "~", " ", "b1"}], "]"}]}], ")"}],
     "~", "MamboTimes", "~", 
    RowBox[{"(", 
     RowBox[{"aframe3", "~", "MamboCross", "~", " ", "b1"}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b3", "=", 
   RowBox[{"b1", "~", "MamboCross", "~", " ", "b2"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"aframe", ",", "asteer", ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<aframe\>\"", "<>", 
         RowBox[{"ToString", "[", "i", "]"}]}], "]"}], "~", "MamboDot", "~", 
       RowBox[{"Symbol", "[", 
        RowBox[{"\"\<b\>\"", "<>", 
         RowBox[{"ToString", "[", "j", "]"}]}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "3"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"j", ",", "3"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"q8", ",", "1"}], "}"}]}], "}"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"Aframe", ",", "Asteer", ",", 
    RowBox[{"{", 
     RowBox[{"aframe", ",", "p3", ",", "p4", ",", "0"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"asteer", ",", "p5", ",", "p6", ",", "0"}], "}"}]}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"{", 
   RowBox[{"Asteer", ",", "Afront", ",", "NullVector"}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"{", 
   RowBox[{"asteer", ",", "afront", ",", "q9", ",", "3"}], "}"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Aframe"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Arear", ",", "Aframe"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Aframe", ",", "Asteer"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Asteer", ",", "Afront"}], "}"}]}], "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"Prear", ",", "Pfront"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Arear", ",", "Prear", ",", "arear", ",", 
     RowBox[{"R", "*", 
      RowBox[{"Cos", "[", "q10", "]"}]}], ",", 
     RowBox[{"R", "*", 
      RowBox[{"Sin", "[", "q10", "]"}]}], ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Afront", ",", "Pfront", ",", "afront", ",", 
     RowBox[{"R", "*", 
      RowBox[{"Cos", "[", "q11", "]"}]}], ",", 
     RowBox[{"R", "*", 
      RowBox[{"Sin", "[", "q11", "]"}]}], ",", "0"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"FindTranslation", "[", 
      RowBox[{"Arear", ",", "Prear"}], "]"}], "~", "MamboDot", "~", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"w", ",", "3"}], "]"}], "~", "MamboCross", "~", " ", 
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"arear", ",", "3"}], "]"}]}], ")"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"W", ",", "Prear"}], "]"}], "~", "MamboDot", "~", " ", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";", 
  RowBox[{"f3", "=", 
   RowBox[{"Simplify", "[", 
    RowBox[{
     RowBox[{"FindTranslation", "[", 
      RowBox[{"Afront", ",", "Pfront"}], "]"}], "~", "MamboDot", "~", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"w", ",", "3"}], "]"}], "~", "MamboCross", "~", " ", 
       RowBox[{"MakeTranslations", "[", 
        RowBox[{"afront", ",", "3"}], "]"}]}], ")"}]}], "]"}]}], ";", 
  RowBox[{"f4", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"W", ",", "Pfront"}], "]"}], "~", "MamboDot", "~", " ", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"w", ",", "3"}], "]"}]}]}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareStates", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Symbol", "[", 
     RowBox[{"\"\<q\>\"", "<>", 
      RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "11"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"q5", "'"}], "\[Equal]", "u1"}], ",", 
       RowBox[{
        RowBox[{"q7", "'"}], "\[Equal]", "u2"}], ",", 
       RowBox[{
        RowBox[{"q8", "'"}], "\[Equal]", "u3"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"DiffTime", "[", "f1", "]"}], "\[Equal]", "0"}], ",", 
       RowBox[{
        RowBox[{"DiffTime", "[", "f3", "]"}], "\[Equal]", "0"}]}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"LinearVelocity", "[", 
            RowBox[{"W", ",", "Arear"}], "]"}], "~", "MamboPlus", "~", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"AngularVelocity", "[", 
              RowBox[{"w", ",", "arear"}], "]"}], "~", "MamboCross", "~", 
             RowBox[{"FindTranslation", "[", 
              RowBox[{"Arear", ",", "Prear"}], "]"}]}], ")"}]}], ")"}], "~", 
         "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"LinearVelocity", "[", 
            RowBox[{"W", ",", "Afront"}], "]"}], "~", "MamboPlus", "~", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"AngularVelocity", "[", 
              RowBox[{"w", ",", "afront"}], "]"}], "~", "MamboCross", "~", 
             RowBox[{"FindTranslation", "[", 
              RowBox[{"Afront", ",", "Pfront"}], "]"}]}], ")"}]}], ")"}], "~",
          "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], ";"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetTime", "[", "t", "]"}], ";", 
  RowBox[{"MotionOutput", "[", 
   RowBox[{
    RowBox[{"\"\<ode\>\"", "->", "kde"}], ",", 
    RowBox[{"\"\<states\>\"", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q1", "\[Rule]", "0"}], ",", 
       RowBox[{"q2", "\[Rule]", "0"}], ",", 
       RowBox[{"q3", "\[Rule]", ".4207354924"}], ",", 
       RowBox[{"q4", "\[Rule]", "0"}], ",", 
       RowBox[{"q5", "\[Rule]", "1"}], ",", 
       RowBox[{"q6", "\[Rule]", 
        RowBox[{".1280053714", "\[Times]", 
         SuperscriptBox["10", 
          RowBox[{"-", "1"}]]}]}], ",", 
       RowBox[{"q7", "\[Rule]", "0"}], ",", 
       RowBox[{"q8", "\[Rule]", ".7"}], ",", 
       RowBox[{"q9", "\[Rule]", "0"}], ",", 
       RowBox[{"q10", "\[Rule]", 
        RowBox[{"-", "1.583596864"}]}], ",", 
       RowBox[{"q11", "\[Rule]", "3.288069682"}]}], "}"}]}], ",", 
    RowBox[{"\"\<parameters\>\"", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"p1", "\[Rule]", 
        RowBox[{"-", ".348968837"}]}], ",", 
       RowBox[{"p2", "\[Rule]", ".937134317"}], ",", 
       RowBox[{"p3", "\[Rule]", "1.296859217"}], ",", 
       RowBox[{"p4", "\[Rule]", ".6774200135"}], ",", 
       RowBox[{"p5", "\[Rule]", 
        RowBox[{"-", ".7601008180"}]}], ",", 
       RowBox[{"p6", "\[Rule]", 
        RowBox[{"-", ".10"}]}], ",", 
       RowBox[{"R", "\[Rule]", ".5"}], ",", 
       RowBox[{"marker", "\[Rule]", ".2"}]}], "}"}]}], ",", 
    RowBox[{"\"\<insignals\>\"", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"u1", "\[Rule]", "0"}], ",", 
       RowBox[{"u2", "\[Rule]", "0"}], ",", 
       RowBox[{"u3", "\[Rule]", "0"}]}], "}"}]}], ",", "\"\<checksings\>\"", 
    ",", 
    RowBox[{"\"\<filename\>\"", "\[Rule]", "\"\<bike.dyn\>\""}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{3.668365092704736*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"v", "=."}], ";", 
  RowBox[{"aframe1", "=."}], ";", 
  RowBox[{"aframe2", "=."}], ";", 
  RowBox[{"aframe3", "=."}], ";", 
  RowBox[{"b1", "=."}], ";", 
  RowBox[{"b2", "=."}], ";", 
  RowBox[{"b3", "=."}], ";", 
  RowBox[{"f1", "=."}], ";", 
  RowBox[{"f2", "=."}], ";", 
  RowBox[{"f3", "=."}], ";", 
  RowBox[{"f4", "=."}], ";", 
  RowBox[{"kde", "=."}], ";"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 10, Section \
10.4", "Section"]], "Text",
 CellChangeTimes->{3.7038596209887753`*^9, 3.703859985179386*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", 
  RowBox[{
  "W", ",", "Base", ",", "Lamp", ",", "Bracket", ",", "UpperBeam", ",", 
   "LowerBeam", ",", "MiddleBeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{
  "W", ",", "Base", ",", "Lamp", ",", "Bracket", ",", "UpperBeam", ",", 
   "LowerBeam", ",", "MiddleBeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{
  "w", ",", "base", ",", "lamp", ",", "bracket", ",", "upperbeam", ",", 
   "lowerbeam", ",", "middlebeam"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineNeighbors", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Base"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "LowerBeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"LowerBeam", ",", "Bracket"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "Lamp"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "MiddleBeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "UpperBeam"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "W", ",", "w"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Base", ",", "Base", ",", "base"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"LowerBeam", ",", "LowerBeam", ",", "lowerbeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"MiddleBeam", ",", "MiddleBeam", ",", "middlebeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"UpperBeam", ",", "UpperBeam", ",", "upperbeam"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "Bracket", ",", "bracket"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Lamp", ",", "Lamp", ",", "lamp"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"W", ",", "Base", ",", "w", ",", "q1", ",", "q2", ",", "q3"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Base", ",", "LowerBeam", ",", "base", ",", "0", ",", "0", ",", "p1"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Base", ",", "MiddleBeam", ",", "base", ",", "0", ",", "0", ",", "p2"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Base", ",", "UpperBeam", ",", "base", ",", "0", ",", "0", ",", "p3"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "LowerBeam", ",", "Bracket", ",", "lowerbeam", ",", "0", ",", "0", ",", 
     "p4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
    "Bracket", ",", "Lamp", ",", "bracket", ",", "0", ",", "0", ",", "p5"}], 
    "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"w", ",", "base", ",", 
     RowBox[{"{", 
      RowBox[{"q4", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q5", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q6", ",", "3"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"base", ",", "lowerbeam", ",", "q7", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"lowerbeam", ",", "bracket", ",", "q10", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"bracket", ",", "lamp", ",", "q11", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"base", ",", "middlebeam", ",", "q8", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"base", ",", "upperbeam", ",", "q9", ",", "2"}], "}"}]}], 
  "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"HUpperBeam", ",", "HBracket"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "UpperBeam", ",", "HUpperBeam", ",", "upperbeam", ",", "0", ",", "0", ",",
      "p6"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"Bracket", ",", "HBracket", ",", "bracket", ",", 
     RowBox[{"-", "p7"}], ",", "0", ",", "p8"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"HBracket", ",", "HUpperBeam"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"base", ",", "1"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f2", "=", 
   RowBox[{
    RowBox[{"FindTranslation", "[", 
     RowBox[{"HBracket", ",", "HUpperBeam"}], "]"}], "~", "MamboDot", "~", 
    RowBox[{"MakeTranslations", "[", 
     RowBox[{"base", ",", "3"}], "]"}]}]}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareStates", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Symbol", "[", 
     RowBox[{"\"\<q\>\"", "<>", 
      RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "11"}], "}"}]}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"LinearVelocity", "[", 
          RowBox[{"W", ",", "Base"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"w", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"w", ",", "base"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"base", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"i", "+", "3"}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"base", ",", "upperbeam"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"base", ",", "2"}], "]"}]}], "\[Equal]", "u7"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"bracket", ",", "lamp"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"base", ",", "2"}], "]"}]}], "\[Equal]", "u8"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"q8", "'"}], "+", 
         RowBox[{"q9", "'"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{
        RowBox[{"DiffTime", "[", "f1", "]"}], "\[Equal]", "0"}], ",", 
       RowBox[{
        RowBox[{"DiffTime", "[", "f2", "]"}], "\[Equal]", "0"}]}], "}"}]}], 
    "]"}]}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"SetTime", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", "kde"}], ",", 
   RowBox[{"\"\<parameters\>\"", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"p1", "\[Rule]", "11"}], ",", 
      RowBox[{"p2", "\[Rule]", "12.5"}], ",", 
      RowBox[{"p3", "\[Rule]", "14"}], ",", 
      RowBox[{"p4", "\[Rule]", "45"}], ",", 
      RowBox[{"p5", "\[Rule]", "3.5"}], ",", 
      RowBox[{"p6", "\[Rule]", "45"}], ",", 
      RowBox[{"p8", "\[Rule]", "2.6"}], ",", 
      RowBox[{"p7", "\[Rule]", "1.5"}]}], "}"}]}], ",", 
   RowBox[{"\"\<states\>\"", "->", 
    RowBox[{"{", 
     RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", 
      RowBox[{"q5", "\[Rule]", "0.01"}], ",", "q6", ",", 
      RowBox[{"q7", "\[Rule]", "1.200039725"}], ",", 
      RowBox[{"q8", "\[Rule]", "1.8"}], ",", 
      RowBox[{"q9", "\[Rule]", "1.2"}], ",", 
      RowBox[{"q10", "\[Rule]", 
       RowBox[{"-", ".6769771893"}]}], ",", 
      RowBox[{"q11", "\[Rule]", "2"}]}], "}"}]}], ",", 
   RowBox[{"\"\<insignals\>\"", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"u1", "\[Rule]", "0"}], ",", 
      RowBox[{"u2", "\[Rule]", "0"}], ",", 
      RowBox[{"u3", "\[Rule]", "0"}], ",", 
      RowBox[{"u4", "\[Rule]", "0"}], ",", 
      RowBox[{"u5", "\[Rule]", "0"}], ",", 
      RowBox[{"u6", "\[Rule]", "0"}], ",", 
      RowBox[{"u7", "\[Rule]", 
       RowBox[{
        RowBox[{"-", ".2"}], 
        RowBox[{"Cos", "[", "t", "]"}]}]}], ",", 
      RowBox[{"u8", "\[Rule]", 
       RowBox[{"Cos", "[", "t", "]"}]}]}], "}"}]}]}], "]"}]}], "Input",
 CellChangeTimes->{{3.6683652112511873`*^9, 3.668365219008956*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"f1", "=."}], ";", 
  RowBox[{"f2", "=."}], ";", 
  RowBox[{"kde", "=."}], ";"}]], "Input"],

Cell[TextData[StyleBox["Mambo Toolbox commands from Chapter 11, Section \
11.1", "Section"]], "Text",
 CellChangeTimes->{{3.703859629835718*^9, 3.7038596304035273`*^9}, 
   3.703859990531506*^9}],

Cell[BoxData[{"Restart", "\[IndentingNewLine]", 
 RowBox[{"DeclareObservers", "[", "N", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclarePoints", "[", 
  RowBox[{"N", ",", "B"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareTriads", "[", 
  RowBox[{"n", ",", "b"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineObservers", "[", 
  RowBox[{"N", ",", "N", ",", "n"}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"N", ",", "B", ",", "n", ",", "q1", ",", "q2", ",", "q3"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefineTriads", "[", 
  RowBox[{"n", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"q4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q6", ",", "3"}], "}"}]}], "]"}], "\[IndentingNewLine]", 
 RowBox[{"DeclareStates", "[", 
  RowBox[{"q1", ",", "q2", ",", "q3", ",", "q4", ",", "q5", ",", "q6"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{"p", "=", 
  RowBox[{"LinearMomentum", "[", 
   RowBox[{"N", ",", "B"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"h", "=", 
  RowBox[{"AngularMomentum", "[", 
   RowBox[{"N", ",", "b"}], "]"}]}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"kde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"LinearVelocity", "[", 
          RowBox[{"N", ",", "B"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"n", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"AngularVelocity", "[", 
          RowBox[{"n", ",", "b"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"b", ",", "i"}], "]"}]}], "\[Equal]", 
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", 
           RowBox[{"i", "+", "3"}], "]"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], ";", 
  RowBox[{"skde", "=", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Solve", "[", 
     RowBox[{"kde", ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q1", "'"}], ",", 
        RowBox[{"q2", "'"}], ",", 
        RowBox[{"q3", "'"}], ",", 
        RowBox[{"q4", "'"}], ",", 
        RowBox[{"q5", "'"}], ",", 
        RowBox[{"q6", "'"}]}], "}"}]}], "]"}], "]"}]}]}]], "Input"],

Cell[BoxData[
 RowBox[{"p", "=", 
  RowBox[{"MamboSimplify", "[", 
   RowBox[{"p", "/.", "skde"}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"h", "=", 
  RowBox[{"MamboSimplify", "[", 
   RowBox[{"h", "/.", "skde"}], "]"}]}]], "Input"],

Cell[BoxData[{
 RowBox[{"DeclareStates", "[", 
  RowBox[{"u1", ",", "u2", ",", "u3", ",", "u4", ",", "u5", ",", "u6"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"DiffTime", "[", 
          RowBox[{"p", ",", "n"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"n", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"DiffTime", "[", 
          RowBox[{"h", ",", "n"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"b", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], ";"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"SetTime", "[", "t", "]"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{"kde", ",", "dde"}], "]"}]}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<q\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "]"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"{", "M", "}"}], ",", 
      RowBox[{"Flatten", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<Inertia\>\"", "<>", 
           RowBox[{"ToString", "[", "i", "]"}], "<>", 
           RowBox[{"ToString", "[", "j", "]"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]}], "]"}]}]}], 
  "]"}]}], "Input",
 CellChangeTimes->{{3.668365245874627*^9, 3.6683652481738567`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"dde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"DiffTime", "[", 
            RowBox[{"p", ",", "n"}], "]"}], "~", "MamboMinus", "~", 
           RowBox[{"MakeTranslations", "[", 
            RowBox[{"n", ",", "0", ",", "0", ",", 
             RowBox[{
              RowBox[{"-", "M"}], " ", "g"}]}], "]"}]}], ")"}], "~", 
         "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"n", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"DiffTime", "[", 
          RowBox[{"h", ",", "n"}], "]"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"b", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{"kde", ",", "dde"}], "]"}]}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<q\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "]"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"M", ",", "g"}], "}"}], ",", 
      RowBox[{"Flatten", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<Inertia\>\"", "<>", 
           RowBox[{"ToString", "[", "i", "]"}], "<>", 
           RowBox[{"ToString", "[", "j", "]"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]}], "]"}]}]}], 
  "]"}]}], "Input"],

Cell[BoxData[{
 RowBox[{"DeclarePoints", "[", "P", "]"}], "\[IndentingNewLine]", 
 RowBox[{"DefinePoints", "[", 
  RowBox[{"B", ",", "P", ",", "b", ",", "p1", ",", "p2", ",", "p3"}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rNP", "=", 
   RowBox[{"FindTranslation", "[", 
    RowBox[{"N", ",", "P"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rBP", "=", 
   RowBox[{"FindTranslation", "[", 
    RowBox[{"B", ",", "P"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dde", "=", 
   RowBox[{"Union", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"DiffTime", "[", 
            RowBox[{"p", ",", "n"}], "]"}], "~", "MamboMinus", "~", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"k", 
               RowBox[{"(", 
                RowBox[{
                 FractionBox["l0", 
                  RowBox[{"MamboNorm", "[", "rNP", "]"}]], "-", "1"}], 
                ")"}]}], ")"}], "~", "MamboTimes", "~", "rNP"}], ")"}]}], 
          ")"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"n", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"DiffTime", "[", 
            RowBox[{"h", ",", "n"}], "]"}], "~", "MamboMinus", "~", 
           RowBox[{"(", 
            RowBox[{"rBP", "~", "MamboCross", "~", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"k", 
                 RowBox[{"(", 
                  RowBox[{
                   FractionBox["l0", 
                    RowBox[{"MamboNorm", "[", "rNP", "]"}]], "-", "1"}], 
                  ")"}]}], ")"}], "~", "MamboTimes", "~", "rNP"}], ")"}]}], 
            ")"}]}], ")"}], "~", "MamboDot", "~", 
         RowBox[{"MakeTranslations", "[", 
          RowBox[{"b", ",", "i"}], "]"}]}], "\[Equal]", "0"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"MotionOutput", "[", 
  RowBox[{
   RowBox[{"\"\<ode\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{"kde", ",", "dde"}], "]"}]}], ",", 
   RowBox[{"\"\<states\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<q\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Symbol", "[", 
         RowBox[{"\"\<u\>\"", "<>", 
          RowBox[{"ToString", "[", "i", "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "]"}]}], ",", 
   RowBox[{"\"\<parameters\>\"", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "M", ",", "g", ",", "p1", ",", "p2", ",", "p3", ",", "k", ",", "l0"}], 
       "}"}], ",", 
      RowBox[{"Flatten", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Symbol", "[", 
          RowBox[{"\"\<Inertia\>\"", "<>", 
           RowBox[{"ToString", "[", "i", "]"}], "<>", 
           RowBox[{"ToString", "[", "j", "]"}]}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "3"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "3"}], "}"}]}], "]"}], "]"}]}], "]"}]}]}], 
  "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"p", "=."}], ";", 
  RowBox[{"h", "=."}], ";", 
  RowBox[{"kde", "=."}], ";", 
  RowBox[{"skde", "=."}], ";", 
  RowBox[{"rNP", "=."}], ";", 
  RowBox[{"rBP", "=."}], ";", 
  RowBox[{"dde", "=."}], ";"}]], "Input",
 CellChangeTimes->{3.66358376398726*^9, 3.668524136427909*^9}]
}, Open  ]]
},
WindowSize->{1151, 1128},
WindowMargins->{{Automatic, 123}, {Automatic, 0}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
ShowSelection->True,
FrontEndVersion->"10.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (October 11, \
2015)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "UTF-8"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 159, 3, 65, "Chapter"],
Cell[742, 27, 1240, 28, 163, "Text"],
Cell[1985, 57, 52, 1, 37, "Input"],
Cell[2040, 60, 516, 12, 49, "Text"],
Cell[2559, 74, 250, 4, 55, "Input"],
Cell[2812, 80, 191, 3, 47, "Text"],
Cell[3006, 85, 90, 1, 55, "Input"],
Cell[3099, 88, 160, 3, 55, "Input"],
Cell[3262, 93, 247, 7, 55, "Input"],
Cell[3512, 102, 107, 2, 37, "Input"],
Cell[3622, 106, 107, 2, 37, "Input"],
Cell[3732, 110, 107, 2, 37, "Input"],
Cell[3842, 114, 332, 8, 37, "Input"],
Cell[4177, 124, 165, 2, 47, "Text"],
Cell[4345, 128, 818, 20, 89, "Input"],
Cell[5166, 150, 163, 2, 47, "Text"],
Cell[5332, 154, 239, 4, 72, "Input"],
Cell[5574, 160, 27, 0, 37, "Input"],
Cell[5604, 162, 112, 2, 37, "Input"],
Cell[5719, 166, 36, 0, 37, "Input"],
Cell[5758, 168, 157, 3, 55, "Input"],
Cell[5918, 173, 221, 7, 55, "Input"],
Cell[6142, 182, 307, 9, 55, "Input"],
Cell[6452, 193, 336, 9, 72, "Input"],
Cell[6791, 204, 408, 10, 89, "Input"],
Cell[7202, 216, 256, 7, 37, "Input"],
Cell[7461, 225, 285, 8, 55, "Input"],
Cell[7749, 235, 98, 2, 37, "Input"],
Cell[7850, 239, 173, 3, 55, "Input"],
Cell[8026, 244, 157, 3, 55, "Input"],
Cell[8186, 249, 250, 6, 55, "Input"],
Cell[8439, 257, 286, 8, 37, "Input"],
Cell[8728, 267, 580, 14, 123, "Input"],
Cell[9311, 283, 157, 3, 55, "Input"],
Cell[9471, 288, 232, 7, 37, "Input"],
Cell[9706, 297, 603, 16, 106, "Input"],
Cell[10312, 315, 302, 9, 37, "Input"],
Cell[10617, 326, 94, 2, 37, "Input"],
Cell[10714, 330, 583, 17, 55, "Input"],
Cell[11300, 349, 135, 3, 37, "Input"],
Cell[11438, 354, 300, 9, 37, "Input"],
Cell[11741, 365, 162, 2, 47, "Text"],
Cell[11906, 369, 1143, 32, 140, "Input"],
Cell[13052, 403, 146, 4, 37, "Input"],
Cell[13201, 409, 300, 9, 37, "Input"],
Cell[13504, 420, 1177, 31, 140, "Input"],
Cell[14684, 453, 235, 7, 37, "Input"],
Cell[14922, 462, 165, 2, 47, "Text"],
Cell[15090, 466, 577, 12, 123, "Input"],
Cell[15670, 480, 91, 2, 37, "Input"],
Cell[15764, 484, 913, 22, 140, "Input"],
Cell[16680, 508, 88, 2, 37, "Input"],
Cell[16771, 512, 91, 2, 37, "Input"],
Cell[16865, 516, 299, 8, 55, "Input"],
Cell[17167, 526, 107, 2, 37, "Input"],
Cell[17277, 530, 347, 9, 72, "Input"],
Cell[17627, 541, 3136, 81, 259, "Input"],
Cell[20766, 624, 2423, 62, 174, "Input"],
Cell[23192, 688, 777, 16, 123, "Input"],
Cell[23972, 706, 196, 5, 37, "Input"],
Cell[24171, 713, 78, 2, 37, "Input"],
Cell[24252, 717, 4701, 138, 123, "Input"],
Cell[28956, 857, 657, 19, 37, "Input"],
Cell[29616, 878, 2528, 74, 106, "Input"],
Cell[32147, 954, 165, 2, 47, "Text"],
Cell[32315, 958, 931, 25, 106, "Input"],
Cell[33249, 985, 95, 3, 37, "Input"],
Cell[33347, 990, 765, 20, 129, "Input"],
Cell[34115, 1012, 95, 3, 37, "Input"],
Cell[34213, 1017, 997, 33, 72, "Input"],
Cell[35213, 1052, 453, 15, 37, "Input"],
Cell[35669, 1069, 94, 3, 37, "Input"],
Cell[35766, 1074, 163, 2, 47, "Text"],
Cell[35932, 1078, 130, 3, 37, "Input"],
Cell[36065, 1083, 128, 3, 37, "Input"],
Cell[36196, 1088, 128, 3, 37, "Input"],
Cell[36327, 1093, 306, 10, 37, "Input"],
Cell[36636, 1105, 189, 6, 37, "Input"],
Cell[36828, 1113, 100, 2, 37, "Input"],
Cell[36931, 1117, 98, 2, 37, "Input"],
Cell[37032, 1121, 69, 1, 37, "Input"],
Cell[37104, 1124, 156, 4, 37, "Input"],
Cell[37263, 1130, 1793, 54, 174, "Input"],
Cell[39059, 1186, 243, 8, 37, "Input"],
Cell[39305, 1196, 195, 3, 47, "Text"],
Cell[39503, 1201, 472, 10, 89, "Input"],
Cell[39978, 1213, 48, 0, 37, "Input"],
Cell[40029, 1215, 122, 3, 37, "Input"],
Cell[40154, 1220, 514, 15, 72, "Input"],
Cell[40671, 1237, 88, 2, 37, "Input"],
Cell[40762, 1241, 68, 1, 37, "Input"],
Cell[40833, 1244, 62, 1, 37, "Input"],
Cell[40898, 1247, 92, 3, 37, "Input"],
Cell[40993, 1252, 1337, 37, 140, "Input"],
Cell[42333, 1291, 125, 3, 37, "Input"],
Cell[42461, 1296, 299, 8, 55, "Input"],
Cell[42763, 1306, 107, 2, 37, "Input"],
Cell[42873, 1310, 330, 8, 72, "Input"],
Cell[43206, 1320, 4223, 117, 276, "Input"],
Cell[47432, 1439, 8056, 240, 429, "Input"],
Cell[55491, 1681, 1272, 30, 89, "Input"],
Cell[56766, 1713, 428, 12, 55, "Input"],
Cell[57197, 1727, 204, 5, 37, "Input"],
Cell[57404, 1734, 902, 25, 72, "Input"],
Cell[58309, 1761, 124, 4, 37, "Input"],
Cell[58436, 1767, 165, 2, 47, "Text"],
Cell[58604, 1771, 2404, 69, 191, "Input"],
Cell[61011, 1842, 3353, 93, 208, "Input"],
Cell[64367, 1937, 224, 6, 37, "Input"],
Cell[64594, 1945, 165, 2, 47, "Text"],
Cell[64762, 1949, 4735, 129, 395, "Input"],
Cell[69500, 2080, 2036, 54, 123, "Input"],
Cell[71539, 2136, 1047, 27, 55, "Input"],
Cell[72589, 2165, 1513, 41, 106, "Input"],
Cell[74105, 2208, 446, 13, 37, "Input"],
Cell[74554, 2223, 165, 2, 47, "Text"],
Cell[74722, 2227, 3448, 91, 225, "Input"],
Cell[78173, 2320, 996, 26, 89, "Input"],
Cell[79172, 2348, 968, 25, 55, "Input"],
Cell[80143, 2375, 1421, 37, 89, "Input"],
Cell[81567, 2414, 162, 4, 37, "Input"],
Cell[81732, 2420, 163, 2, 47, "Text"],
Cell[81898, 2424, 282, 6, 72, "Input"],
Cell[82183, 2432, 197, 6, 37, "Input"],
Cell[82383, 2440, 161, 4, 37, "Input"],
Cell[82547, 2446, 248, 6, 55, "Input"],
Cell[82798, 2454, 692, 16, 106, "Input"],
Cell[83493, 2472, 91, 2, 37, "Input"],
Cell[83587, 2476, 127, 3, 37, "Input"],
Cell[83717, 2481, 665, 15, 106, "Input"],
Cell[84385, 2498, 90, 2, 37, "Input"],
Cell[84478, 2502, 90, 2, 37, "Input"],
Cell[84571, 2506, 165, 2, 47, "Text"],
Cell[84739, 2510, 1100, 26, 174, "Input"],
Cell[85842, 2538, 1330, 34, 174, "Input"],
Cell[87175, 2574, 1538, 37, 191, "Input"],
Cell[88716, 2613, 1493, 37, 191, "Input"],
Cell[90212, 2652, 65, 2, 37, "Input"],
Cell[90280, 2656, 165, 2, 47, "Text"],
Cell[90448, 2660, 975, 29, 79, "Input"],
Cell[91426, 2691, 315, 10, 37, "Input"],
Cell[91744, 2703, 382, 10, 37, "Input"],
Cell[92129, 2715, 131, 3, 37, "Input"],
Cell[92263, 2720, 3314, 89, 259, "Input"],
Cell[95580, 2811, 407, 9, 55, "Input"],
Cell[95990, 2822, 1046, 29, 55, "Input"],
Cell[97039, 2853, 129, 3, 37, "Input"],
Cell[97171, 2858, 163, 2, 47, "Text"],
Cell[97337, 2862, 1301, 32, 174, "Input"],
Cell[98641, 2896, 930, 23, 106, "Input"],
Cell[99574, 2921, 1965, 55, 89, "Input"],
Cell[101542, 2978, 1362, 38, 89, "Input"],
Cell[102907, 3018, 1835, 51, 89, "Input"],
Cell[104745, 3071, 2582, 72, 140, "Input"],
Cell[107330, 3145, 447, 11, 72, "Input"],
Cell[107780, 3158, 192, 3, 47, "Text"],
Cell[107975, 3163, 4627, 127, 378, "Input"],
Cell[112605, 3292, 2036, 54, 123, "Input"],
Cell[114644, 3348, 2344, 64, 123, "Input"],
Cell[116991, 3414, 1854, 47, 141, "Input"],
Cell[118848, 3463, 409, 13, 37, "Input"],
Cell[119260, 3478, 167, 2, 47, "Text"],
Cell[119430, 3482, 3340, 89, 208, "Input"],
Cell[122773, 3573, 996, 26, 89, "Input"],
Cell[123772, 3601, 2293, 62, 106, "Input"],
Cell[126068, 3665, 1702, 41, 89, "Input"],
Cell[127773, 3708, 125, 4, 37, "Input"],
Cell[127901, 3714, 195, 3, 47, "Text"],
Cell[128099, 3719, 1161, 27, 191, "Input"],
Cell[129263, 3748, 1495, 43, 72, "Input"],
Cell[130761, 3793, 117, 3, 37, "Input"],
Cell[130881, 3798, 117, 3, 37, "Input"],
Cell[131001, 3803, 976, 27, 72, "Input"],
Cell[131980, 3832, 1482, 40, 89, "Input"],
Cell[133465, 3874, 2483, 71, 106, "Input"],
Cell[135951, 3947, 3726, 104, 280, "Input"],
Cell[139680, 4053, 311, 9, 37, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
